#region Copyright Robert Bosch GmbH, 2012.
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// File name  $Workfile:   ApiDemonstratorMainForm.cs  $
//            $Revision:   1.0  $
// Creation Date       : 25 Dec 2011
// First Author        : msn4kor
//      Last Save $Date:   Apr 20 2012 $
//            $Revision:   1.1 [Discussion settings were not in sink with Opr client] $
//         Last $Author:   mmo4kor  $
//      Last Save $Date:   Apr 23 2012 $
//            $Revision:   1.2 [Discussion settings were not in sink with Opr client] $
//         Last $Author:   mmo4kor  $
//      Last Save $Date:   May 10 2012 $
//            $Revision:   1.3 [SpeakNowBySeatIds Method Added] $
//         Last $Author:   mmo4kor  $
//      Last Save $Date:   May 29 2012 $
//            $Revision:   1.4 [Rework after API changes] $
//         Last $Author:   msn4kor  $
//      Last Save $Date:   Aug 03 2012 $
//            $Revision:   1.5 [Retrieve Microphone Status Method Added] $
//         Last $Author:   mmo4kor  $
//
//      Description         :The DcnApiDemonstrator application is a PC-based software component 
//                              communicating with the DCN SW Server. The DcnApiDemonstrator application 
//                              provides a user interface to configure and control the DCNNG system
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Bosch.Dcn.Ecpc.Client.Api.Interfaces;
using Bosch.Dcn.Ecpc.Client.Api.Logic;
using System.Text.RegularExpressions;
using System.IO;
using System.Data.OleDb;

namespace DcnApiDemonstrator
{
    /// <summary>
    /// Application showing the use of the DCN-SW API for 3rd-party configuration purposes.
    /// </summary>
    public partial class ApiDemonstratorMainForm : Form
    {

        #region Contructor
        public ApiDemonstratorMainForm()
        {
            InitializeComponent();
        }
        #endregion

        #region Initialize
        private void APIDemonstratorMainForm_Load(object sender, EventArgs e)
        {
            configApi = DcnApi.ConfigApi;
            controlApi = DcnApi.ControlApi;

            //Register for all events
            configApi.AvailabilityChange += OnApiAvailabilityChange;
            configApi.AuthorizationChange += OnApiAuthorizationChange;
            controlApi.AuthorizationChange += OnControlApiAuthorizationChange;

            controlApi.MeetingApi.MeetingStart += OnMeetingStart;
            controlApi.MeetingApi.MeetingStop += OnMeetingStop;
            controlApi.MeetingApi.SessionStart += OnSessionStart;
            controlApi.MeetingApi.SessionStop += OnSessionStop;
            configApi.MeetingApi.MeetingDeletion += OnMeetingDeletion;
            configApi.MeetingApi.MeetingStore += OnMeetingStore;
            configApi.MeetingApi.MeetingUpdate += onMeetingUpdate;
            configApi.MeetingApi.SortPositionChanged += OnSessionOrderUpdate;
            
            configApi.MeetingApi.SessionUpdate += onSessionUpdate;
            configApi.MeetingGroupApi.MeetingGroupStore += onNewMeetingGroupStore;
            configApi.MeetingGroupApi.MeetingGroupUpdate += onMeetingGroupUpdate;
            configApi.DelegateApi.RequestStore += onRequestStore;
            configApi.DelegateApi.RequestUpdate += onRequestUpdate;
            configApi.DelegateApi.RequestDeletion += onRequestDeletion;


            controlApi.DiscussionApi.SpeakersListUpdated += OnSpeakerListUpdated;
            controlApi.DiscussionApi.RequestToSpeakListUpdated += OnRequestListUpdated;
            controlApi.DiscussionApi.RequestToRespondListUpdated += OnResposneRequestListUpdated;
            controlApi.DiscussionApi.RespondListUpdated += OnRespondListUpdated;
            controlApi.DiscussionApi.NotebookersListUpdated += OnNotebookerListUpdated;
            controlApi.DiscussionApi.GroupSpeechTimeUpdated += OnGroupSpeechTimeUpdated;
            controlApi.DiscussionApi.DiscussionSettingsUpdate += OnDiscussionSettingsUpdate;
            controlApi.DiscussionApi.ParticipantTimeLeftUpdated += OnParticipantTimeLeftUpdated;

            configApi.DelegateApi.DelegateDeletion += OnDelegateDeletion;
            configApi.DelegateApi.DelegateStore += OnDelegateStore;
            configApi.DelegateApi.DelegateRegistration += OnDelegateRegistration;
            configApi.DelegateApi.DelegateRegistrationUpdate += OnDelegateRegistrationUpdate;
            configApi.DelegateApi.DelegateUnregistration += OnDelegateUnregistration;
            configApi.DelegateApi.DelegateUpdate += OnDelegateUpdate;

            configApi.MeetingGroupApi.MeetingGroupStore += OnMeetingGroupStore;
            configApi.MeetingGroupApi.MeetingGroupUpdate += OnMeetingGroupUpdate;
            configApi.MeetingGroupApi.MeetingGroupDeletion += OnMeetingGroupDeletion;

            configApi.VoteApi.VoteStore += OnVotingStore;
            configApi.VoteApi.VoteDeletion += OnVoteDeletion;

            controlApi.DiscussionApi.MicOff += OnMicOff;
            controlApi.DiscussionApi.MicOn += OnMicOn;

            controlApi.VoteApi.VotingStart += OnVotingStart;
            controlApi.VoteApi.VotingStop += OnVotingStop;
            controlApi.VoteApi.VotingHold += OnVotingHold;
            controlApi.VoteApi.VotingContinue += OnVotingContinue;
            controlApi.VoteApi.VotingSelect += OnVotingSelect;
            controlApi.VoteApi.VotingTimer += OnVotingTimer;
            controlApi.VoteApi.VotingAccepted += OnVotingAccepted;
            controlApi.VoteApi.VotingRejected += OnVotingRejected;
            controlApi.VoteApi.VotingQuorumUpdated += OnVotingQuorumUpdated;

            controlApi.DcnSystemApi.masterMuteChange += onMaterMuteChange;
            controlApi.DcnSystemApi.masterVolumeChange += onMasterVolumeChange;

            controlApi.DiscussionApi.ChairmanPriorityListUpdated += onNumberofActivePriosChange;

            configApi.SeatDataApi.SeatUpdate += onSeatUpdate;

            controlApi.MeetingApi.AttendanceStart += onAttendanceStart;
            controlApi.MeetingApi.AttendanceStop+= onAttendanceStop;


            // Update the main form...
            UpdateApiDemonstratorMainForm();
            RetrieveDelegates();
        }

        private void UpdateApiDemonstratorMainForm()
        {
            UpdateAvailabilityStatus();
            btnConnectAndDisconnect.Enabled = true;
        }
        #endregion

        #region TearDown
        private void APIDemonstratorMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Unregister for all event registrations
            configApi.AvailabilityChange -= OnApiAvailabilityChange;
            configApi.AuthorizationChange -= OnApiAuthorizationChange;
            configApi.AuthorizationChange -= OnControlApiAuthorizationChange;

            controlApi.MeetingApi.MeetingStart -= OnMeetingStart;
            controlApi.MeetingApi.MeetingStop -= OnMeetingStop;
            controlApi.MeetingApi.SessionStart -= OnSessionStart;
            controlApi.MeetingApi.SessionStop -= OnSessionStop;
            configApi.MeetingApi.MeetingDeletion -= OnMeetingDeletion;
            configApi.MeetingApi.MeetingStore -= OnMeetingStore;
            configApi.MeetingApi.MeetingUpdate -= onMeetingUpdate;
            configApi.MeetingApi.SessionUpdate -= onSessionUpdate;
            configApi.MeetingGroupApi.MeetingGroupStore -= onNewMeetingGroupStore;
            configApi.MeetingGroupApi.MeetingGroupUpdate -= onMeetingGroupUpdate;
            configApi.DelegateApi.RequestStore -= onRequestStore;
            configApi.DelegateApi.RequestUpdate -= onRequestUpdate;
            configApi.DelegateApi.RequestDeletion -= onRequestDeletion;

            controlApi.DiscussionApi.SpeakersListUpdated -= OnSpeakerListUpdated;
            controlApi.DiscussionApi.RequestToSpeakListUpdated -= OnRequestListUpdated;
            controlApi.DiscussionApi.RequestToRespondListUpdated -= OnResposneRequestListUpdated;
            controlApi.DiscussionApi.NotebookersListUpdated -= OnNotebookerListUpdated;
            controlApi.DiscussionApi.RespondListUpdated -= OnRespondListUpdated;

            controlApi.DiscussionApi.GroupSpeechTimeUpdated -= OnGroupSpeechTimeUpdated;
            controlApi.DiscussionApi.DiscussionSettingsUpdate -= OnDiscussionSettingsUpdate;

            configApi.DelegateApi.DelegateDeletion -= OnDelegateDeletion;
            configApi.DelegateApi.DelegateStore -= OnDelegateStore;
            configApi.DelegateApi.DelegateRegistration -= OnDelegateRegistration;
            configApi.DelegateApi.DelegateRegistrationUpdate -= OnDelegateRegistrationUpdate;
            configApi.DelegateApi.DelegateUnregistration -= OnDelegateUnregistration;
            configApi.DelegateApi.DelegateUpdate -= OnDelegateUpdate;

            configApi.MeetingGroupApi.MeetingGroupStore -= OnMeetingGroupStore;
            configApi.MeetingGroupApi.MeetingGroupUpdate -= OnMeetingGroupUpdate;
            configApi.MeetingGroupApi.MeetingGroupDeletion -= OnMeetingGroupDeletion;

            controlApi.DiscussionApi.MicOff -= OnMicOff;
            controlApi.DiscussionApi.MicOn -= OnMicOn;

            configApi.VoteApi.VoteStore -= OnVotingStore;
            configApi.VoteApi.VoteDeletion -= OnVoteDeletion;

            controlApi.VoteApi.VotingStart -= OnVotingStart;
            controlApi.VoteApi.VotingStop -= OnVotingStop;
            controlApi.VoteApi.VotingHold -= OnVotingHold;
            controlApi.VoteApi.VotingContinue -= OnVotingContinue;
            controlApi.VoteApi.VotingSelect -= OnVotingSelect;
            controlApi.VoteApi.VotingTimer -= OnVotingTimer;
            controlApi.VoteApi.VotingAccepted -= OnVotingAccepted;
            controlApi.VoteApi.VotingRejected -= OnVotingRejected;
            controlApi.VoteApi.VotingQuorumUpdated -= OnVotingQuorumUpdated;

            controlApi.DcnSystemApi.masterMuteChange -= onMaterMuteChange;
            controlApi.DcnSystemApi.masterVolumeChange -= onMasterVolumeChange;

            controlApi.DiscussionApi.ChairmanPriorityListUpdated -= onNumberofActivePriosChange;

            configApi.SeatDataApi.SeatUpdate -= onSeatUpdate;
            controlApi.MeetingApi.AttendanceStart -= onAttendanceStart;
            controlApi.MeetingApi.AttendanceStart -= onAttendanceStop;
            // Check if the API was initialized from this application, if so terminate it first...
            if (isApiDemonstratorInitialized)
            {
                TerminateApiConnection();
            }
            DelegateDataset.Dispose();
            DelegateDatagridview.Dispose();
            configApi.Dispose();
            controlApi.Dispose();
            DelegateDataset = null;
            DelegateDatagridview = null;
            configApi = null;
            controlApi = null;
        }

        private void TerminateApiConnection()
        {
            if (configApi != null)
            {
                configApi.Terminate();
            }
            if (controlApi != null)
            {
                controlApi.Terminate();
            }
        }
        #endregion

        #region Handlers registered with the DCN-SW API

        // Use the handlers below to update status to the user interface!

        private void onAttendanceStart(object sender, MeetingControlEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingControlEventArgs>(onAttendanceStart), sender, e);
            }
            else
            {
                LogNotification("Attendance started");
                LogFeedback("Attendance started");
            }
        }

        private void onAttendanceStop(object sender, MeetingControlEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingControlEventArgs>(onAttendanceStop), sender, e);
            }
            else
            {
                LogNotification("Attendance stopped");
                LogFeedback("Attendance stopped");
            }
        }

        private void onRequestStore(object sender, RequestDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<RequestDataChangedEventArgs>(onRequestStore), sender, e);
            }
            else
            {
                LogNotification("onRequestStore session id " + Properties.Resources.Comma + e.SessionIds[0].ToString());
            }
        }

        private void onRequestUpdate(object sender, RequestDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<RequestDataChangedEventArgs>(onRequestUpdate), sender, e);
            }
            else
            {
                LogNotification("onRequestUpdate session id  " + Properties.Resources.Comma + e.SessionIds[0].ToString());
            }
        }

        private void onRequestDeletion(object sender, RequestDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<RequestDataChangedEventArgs>(onRequestDeletion), sender, e);
            }
            else
            {
                LogNotification("onRequestDeletion session id  " + Properties.Resources.Comma + e.SessionIds[0].ToString());
            }
        }


        private void onNewMeetingGroupStore(object sender, MeetingGroupDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingGroupDataChangedEventArgs>(onNewMeetingGroupStore), sender, e);
            }
            else
            {
                LogNotification("onMeetingGroupStore " + Properties.Resources.Comma + e.ConfigIds.ToString());
            }
        }

        private void onMeetingGroupUpdate(object sender, MeetingGroupDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingGroupDataChangedEventArgs>(onMeetingGroupUpdate), sender, e);
            }
            else
            {
                LogNotification("onMeetingGroupUpdate " + Properties.Resources.Comma + e.ConfigIds.ToString());
            }
        }

        private void onMasterVolumeChange(object sender, CountEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<CountEventArgs>(onMasterVolumeChange), sender, e);
            }
            else
            {
                LogNotification("onMasterVolumeChange" + Properties.Resources.Comma + e.Count.ToString());
            }
        }
        private void onMaterMuteChange(object sender, OnOffEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<OnOffEventArgs>(onMaterMuteChange), sender, e);
            }
            else
            {
                LogNotification("onMaterMuteChange" + Properties.Resources.Comma + e.OnOff.ToString());
            }
        }

        private void onNumberofActivePriosChange(object sender, ChairmanPriorityListUpdatedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<ChairmanPriorityListUpdatedEventArgs>(onNumberofActivePriosChange), sender, e);
            }
            else
            {
                LogNotification("OnChairmanPriorityListUpdated" + Properties.Resources.Comma + " Key Prios = " + e.Keyprio + " Virtual Key Prios = " + e.Virtualkeyprio);
            }
        }



        private void OnVotingQuorumUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingQuorumUpdated), sender, e);
            }
            else
            {
                LogNotification("OnVotingQuorumUpdated" + Properties.Resources.Comma + " Voting Quorum Updated");
            }
        }

        private void OnVotingRejected(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingRejected), sender, e);
            }
            else
            {
                LogNotification("OnVotingRejected" + Properties.Resources.Comma + " Voting Rejected");
            }
        }

        private void OnVotingAccepted(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingAccepted), sender, e);
            }
            else
            {
                LogNotification("OnVotingAccepted" + Properties.Resources.Comma + " Voting Accepted");
            }
        }

        private void OnVotingSelect(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingSelect), sender, e);
            }
            else
            {
                LogNotification("OnVotingSelect" + Properties.Resources.Comma + " Voting Selected");
            }
        }

        private void OnVotingContinue(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingContinue), sender, e);
            }
            else
            {
                LogNotification("OnVotingContinue" + Properties.Resources.Comma + " Voting Continue");
            }
        }

        private void OnVotingHold(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingHold), sender, e);
            }
            else
            {
                LogNotification("OnVotingHold" + Properties.Resources.Comma + " Voting put on Hold");
            }
        }

        private void OnVotingStop(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingStop), sender, e);
            }
            else
            {
                LogNotification("OnVotingStop" + Properties.Resources.Comma + " Voting Stopped");
            }
        }

        private void OnVotingStart(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnVotingStart), sender, e);
            }
            else
            {
                LogNotification(" OnVotingStart" + Properties.Resources.Comma + " Voting With Id" + Properties.Resources.Comma + Properties.Resources.Comma + " started");
            }
        }

        private void OnVotingTimer(object sender, VotingTimerControlEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new VotingTimerControlEventHandler(OnVotingTimer), sender, e);
            }
            else
            {
                LogNotification(" OnVotingTimer" + Properties.Resources.Comma + " Voting Time Left  " + e.VotingTimer.VotingTimeLeft.ToString());
            }
        }


        private void OnVotingStore(object sender, VoteDataChangedEventArgs args)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<VoteDataChangedEventArgs>(OnVotingStore), sender, args);
            }
            else
            {
                LogNotification(" OnVotingStore" + Properties.Resources.Comma + " Vote Stored");
            }
        }

        private void OnVoteDeletion(object sender, VoteDataChangedEventArgs args)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<VoteDataChangedEventArgs>(OnVoteDeletion), sender, args);
            }
            else
            {
                LogNotification(" OnVoteDeletion" + Properties.Resources.Comma + " Vote Deleted");
            }
        }

        private void OnMeetingStart(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnMeetingStart), sender, e);
            }
            else
            {
                UpdateDiscussionSettings();
                LogNotification(" OnMeetingStart" + Properties.Resources.Comma + " Meeting With Id" + Properties.Resources.Comma + ActiveMeetingId().ToString()
                    + Properties.Resources.Comma + " started");
            }
        }

        private void OnMeetingStop(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnMeetingStop), sender, e);
            }
            else
            {
                LogNotification("OnMeetingStop" + Properties.Resources.Comma + " Meeting Stopped");
            }
        }

        private void OnSessionStart(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnSessionStart), sender, e);
            }
            else
            {
                UpdateDiscussionSettings();
                LogNotification("OnSessionStart" + Properties.Resources.Comma + " Session With Id" + Properties.Resources.Comma + ActiveSessionId().ToString()
                   + Properties.Resources.Comma + " started");
            }
        }

        private void OnSessionStop(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnSessionStop), sender, e);
            }
            else
            {
                LogNotification("OnSessionStop" + Properties.Resources.Comma + " Session Stopped");
            }
        }

        private void OnMeetingStore(object sender, MeetingDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingDataChangedEventArgs>(OnMeetingStore), sender, e);
            }
            else
            {
                LogNotification("OnMeetingStore" + Properties.Resources.Comma + e.ConfigIds.Length + Properties.Resources.Comma + " Meetings Added");
            }
        }

        private void OnSessionOrderUpdate(object sender, SessionDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<SessionDataChangedEventArgs>(OnSessionOrderUpdate), sender, e);
            }
            else
            {
                LogNotification("OnSessionOrderUpdate" + Properties.Resources.Comma + e.ConfigIds.Length + Properties.Resources.Comma + " Session Order Updated");
            }
        }
        private void OnMeetingDeletion(object sender, MeetingDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingDataChangedEventArgs>(OnMeetingDeletion), sender, e);
            }
            else
            {
                LogNotification("OnMeetingDeletion" + Properties.Resources.Comma + e.ConfigIds.Length + Properties.Resources.Comma + " Meetings Deleted");
            }
        }



        private void OnSpeakerListUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnSpeakerListUpdated), sender, e);
            }
            else
            {
                LogNotification("OnSpeakerListUpdated" + Properties.Resources.Comma + "Speaker List Updated");
            }
        }

        private void OnSpeechTimerInterrupt(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnSpeechTimerInterrupt), sender, e);
            }
            else
            {
                LogNotification("OnSpeechTimerInterrupt" + Properties.Resources.Comma + "Speech Timer Unterrupted");
            }
        }

        private void OnSpeechTimerResume(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnSpeechTimerResume), sender, e);
            }
            else
            {
                LogNotification("OnSpeechTimerInterrupt" + Properties.Resources.Comma + "Speech Timer Interrupted");
            }
        }


        private void OnResposneRequestListUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnResposneRequestListUpdated), sender, e);
            }
            else
            {
                LogNotification("OnResposneRequestListUpdated" + Properties.Resources.Comma + "Comment Request List Updated");
            }
        }

        private void OnRespondListUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnRespondListUpdated), sender, e);
            }
            else
            {
                LogNotification("OnRespondListUpdated" + Properties.Resources.Comma + "Comment speaker List Updated");
            }
        }


        private void OnSpecialStationListUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnSpecialStationListUpdated), sender, e);
            }
            else
            {
                LogNotification("OnSpecialStationListUpdated" + Properties.Resources.Comma + "Special Station List Updated");
            }
        }

        private void OnGroupSpeechTimeUpdated(object sender, GroupTimerControlEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new GroupTimerControlEventHandler(OnGroupSpeechTimeUpdated), sender, e);
            }
            else
            {
                LogNotification("OnGroupSpeechTimeUpdated" + Properties.Resources.Comma + "GroupId: " + e.GroupTimer.GroupId + " RemainingTime :" + e.GroupTimer.GroupTimerLimitInMillis);
            }
        }

        private void OnParticipantTimeLeftUpdated(object sender, TimeLeftControlEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new TimeLeftControlEventHandler(OnParticipantTimeLeftUpdated), sender, e);
            }
            else
            {
                foreach (SPEAKER_TIMELEFT_INFO s in e.SpeakerTimeLeft)
                {
                    LogNotification("OnParticipantTimeLeftUpdated" + Properties.Resources.Comma + " ParticipantId: " + s.ParticipantId.ToString() + ", TimeLeft :" + s.TimeLeft.ToString());
                }
            }
        }

        private void OnNotebookerListUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnNotebookerListUpdated), sender, e);
            }
            else
            {
                LogNotification("OnNotebookerListUpdated" + Properties.Resources.Comma + "Notebooker List Updated");
            }
        }


        private void OnRequestListUpdated(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnRequestListUpdated), sender, e);
            }
            else
            {
                LogNotification("OnRequestListUpdated" + Properties.Resources.Comma + "Request List Updated");
            }
        }

        private void OnMicOff(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnMicOff), sender, e);
            }
            else
            {
                LogNotification("OnMicOff" + Properties.Resources.Comma + "Microphone is off.");
            }
        }

        private void OnMicOn(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnMicOn), sender, e);
            }
            else
            {
                LogNotification("OnMicOn" + Properties.Resources.Comma + "Microphone is on.");
            }
        }

        private void OnDiscussionSettingsUpdate(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnDiscussionSettingsUpdate), sender, e);
            }
            else
            {
                UpdateDiscussionSettings();
                LogNotification("OnDiscussionSettingsUpdate" + Properties.Resources.Comma + "DiscussionSettings were updated");
            }
        }

        private void OnApiAvailabilityChange(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnApiAvailabilityChange), sender, e);
            }
            else
            {
                IConfigApi config = sender as IConfigApi;
                LogNotification("OnApiAvailabilityChange" + Properties.Resources.Comma + "The connection status to the DCN-SW server has changed to"
                                   + Properties.Resources.Comma + config.IsAvailable.ToString());
                isApiAvailable = configApi.IsAvailable;
                UpdateAvailabilityStatus();

            }
        }

        private void OnApiAuthorizationChange(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnApiAuthorizationChange), sender, e);
            }
            else
            {
                IConfigApi config = sender as IConfigApi;
                LogNotification("OnApiAuthorizationChange" + Properties.Resources.Comma + "The authorization status has changed to"
                                    + Properties.Resources.Comma + config.IsAvailable.ToString());
                UpdateAvailabilityStatus();
            }
        }

        private void OnControlApiAuthorizationChange(object sender, EventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler(OnControlApiAuthorizationChange), sender, e);
            }
            else
            {
                IControlApi control = sender as IControlApi;
                LogNotification("OnControlApiAuthorizationChange" + Properties.Resources.Comma + "The authorization status has changed to"
                                    + Properties.Resources.Comma + control.IsAvailable.ToString());
                UpdateAvailabilityStatus();
            }
        }

        private void OnDelegateStore(object sender, DelegateDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<DelegateDataChangedEventArgs>(OnDelegateStore), sender, e);
            }
            else
            {
                LogNotification("OnDelegateStore" + Properties.Resources.Comma + e.ConfigIds[0] + Properties.Resources.Comma + " Delegate stored");
            }
        }

        private void OnDelegateUpdate(object sender, DelegateDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<DelegateDataChangedEventArgs>(OnDelegateUpdate), sender, e);
            }
            else
            {
                LogNotification("OnDelegateUpdate" + Properties.Resources.Comma + e.ConfigIds[0] + Properties.Resources.Comma + " Delegate Updated");
            }
        }

        private void OnMeetingGroupStore(object sender, MeetingGroupDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingGroupDataChangedEventArgs>(OnMeetingGroupStore), sender, e);
            }
            else
            {
                LogNotification("OnMeetingGroupStore" + Properties.Resources.Comma + " Meeting Group stored.");
            }
        }

        private void OnMeetingGroupUpdate(object sender, MeetingGroupDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingGroupDataChangedEventArgs>(OnMeetingGroupUpdate), sender, e);
            }
            else
            {
                LogNotification("OnMeetingGroupUpdate" + Properties.Resources.Comma + " Meeting Group Updated.");
            }
        }

        private void OnMeetingGroupDeletion(object sender, MeetingGroupDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingGroupDataChangedEventArgs>(OnMeetingGroupDeletion), sender, e);
            }
            else
            {
                LogNotification("OnMeetingGroupDeletion" + Properties.Resources.Comma + " Meeting Group deleted");
            }
        }

        private void OnDelegateDeletion(object sender, DelegateDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<DelegateDataChangedEventArgs>(OnDelegateDeletion), sender, e);
            }
            else
            {
                LogNotification("OnDelegateDeletion" + Properties.Resources.Comma + e.ConfigIds[0]+ Properties.Resources.Comma + " Delegate Deleted");
            }
        }

        private void OnDelegateRegistration(object sender, DelegateRegistrationEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<DelegateRegistrationEventArgs>(OnDelegateRegistration), sender, e);
            }
            else
            {
                // TODO: Handle delegate registration here...
            }
        }

        private void OnDelegateRegistrationUpdate(object sender, DelegateRegistrationEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<DelegateRegistrationEventArgs>(OnDelegateRegistrationUpdate), sender, e);
            }
            else
            {
                // TODO: Handle delegate update here...
            }
        }

        private void OnDelegateUnregistration(object sender, DelegateRegistrationEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<DelegateRegistrationEventArgs>(OnDelegateUnregistration), sender, e);
            }
            else
            {
                // TODO: Handle delegate unregistration here...
            }
        }

        private void onMeetingUpdate(object sender, MeetingDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<MeetingDataChangedEventArgs>(onMeetingUpdate), sender, e);
            }
            else
            {
                LogNotification("onMeetingUpdate" + Properties.Resources.Comma + " Meeting updated");
            }
        }

        private void onSessionUpdate(object sender, SessionDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<SessionDataChangedEventArgs>(onSessionUpdate), sender, e);
            }
            else
            {
                LogNotification("onSessionUpdate" + Properties.Resources.Comma + " session updated");
            }
        }

        private void onSeatUpdate(object sender, SeatDataChangedEventArgs e)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<SeatDataChangedEventArgs>(onSeatUpdate), sender, e);
            }
            else
            {
                LogNotification("onSeatUpdate" + Properties.Resources.Comma + " seat updated , Seat id =" + e.ConfigIds[0] );
            }
        }


        #endregion

        #region Handlers for Api function button click

        private void btnSelectVote_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
                 + Properties.Resources.Comma + "SelectVotingById" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {

                    int votingId = 0;
                    API_ERROR error = API_ERROR.ERROR;

                    if (textBoxVotingId.Text.Length != 0)
                    {
                        // Validate session id;
                        if (CheckAndConvertStringValueToInt(textBoxVotingId.Text, out votingId))
                        {
                            error = controlApi.VoteApi.SelectVotingById(votingId);
                            if (error == API_ERROR.NONE)
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnHoldVote_Click(object sender, EventArgs e)
        {
            HoldVoting();
        }

        private void btnResumeVote_Click(object sender, EventArgs e)
        {
            ResumeVoting();
        }

        private void btnStopVoting_Click(object sender, EventArgs e)
        {
            StopVoting();
        }

        private void btnStartVotingById_Click(object sender, EventArgs e)
        {
            int votingId = 0;
            if (textBoxVotingId.Text.Length != 0)
            {
                // Validate session id;
                if (CheckAndConvertStringValueToInt(textBoxVotingId.Text, out votingId))
                {
                    StartVoting(votingId);
                }
            }
        }

        private void btnConnectAndDisconnect_Click(object sender, EventArgs e)
        {
            Button btnSender = sender as Button;
            if (btnSender.Text == Properties.Resources.MainForm_btnConnectText)
            {
                try
                {
                    // Start setting up connection to the DCN-SW server asynchronously.
                    AsyncConnector configApiconnect = new AsyncConnector(configApi.Initialize);
                    AsyncConnector ControlApiConnect = new AsyncConnector(controlApi.Initialize);

                    string ip = "tcp://" + txtBoxIPAddress.Text + ":" + Properties.Settings.Default.portNumber;
                    string username = txtBoxUsername.Text;
                    string password = txtBoxPassword.Text;

                    configApiconnect.BeginInvoke(
                        ip,
                        username,
                        password,
                        new AsyncCallback(ConfigApiConnectorCallback),
                        configApiconnect);

                    ControlApiConnect.BeginInvoke(
                        ip,
                        username,
                        password,
                        new AsyncCallback(ControlApiConnectorCallback),
                        ControlApiConnect);

                    isInConnection = true;
                    btnConnectAndDisconnect.Enabled = false;
                    
                }
                catch (Exception ex)
                {
                    LogFeedback(Properties.Resources.Comma + Properties.Resources.Comma + "Initialize" + Properties.Resources.Comma
                                + Properties.Resources.Comma + ex.Message);
                }
            }
            else
            {
                isInConnection = false;
                btnSender.Text = Properties.Resources.MainForm_btnConnectText;
                TerminateApiConnection();
                isApiDemonstratorInitialized = false;
                UpdateAvailabilityStatus();
            }
        }

        private void ControlApiConnectorCallback(IAsyncResult ar)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new AsyncCallback(ControlApiConnectorCallback), ar);
            }
            else
            {
                AsyncConnector connector = ar.AsyncState as AsyncConnector;
                API_ERROR returnValue = connector.EndInvoke(ar);
                string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.ApiInterface
                                    + Properties.Resources.Comma + "Initialize" + Properties.Resources.Comma + apiErrorString;

                if (API_ERROR.NONE == returnValue)
                {
                    // Triggering setup of connection successful...
                    if (isInConnection)
                    {
                        isApiDemonstratorInitialized = true;
                        UpdateAvailabilityStatus();
                        btnConnectAndDisconnect.Enabled = true;
                        LogFeedback(logString + returnValue.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + returnValue.ToString());
                    }
                }
                else
                {
                    btnConnectAndDisconnect.Text = Properties.Resources.MainForm_btnDisconnectText;
                    UpdateAvailabilityStatus();
                    btnConnectAndDisconnect.Enabled = true;
                    LogFeedback(logString + returnValue.ToString());
                }
            }
        }

        private void ConfigApiConnectorCallback(IAsyncResult ar)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new AsyncCallback(ConfigApiConnectorCallback), ar);
            }
            else
            {
                AsyncConnector connector = ar.AsyncState as AsyncConnector;
                API_ERROR returnValue = connector.EndInvoke(ar);
                string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.ApiInterface
                                    + Properties.Resources.Comma + "Initialize" + Properties.Resources.Comma + apiErrorString;

                if (API_ERROR.NONE == returnValue)
                {
                    // Triggering setup of connection successful...
                    if (isInConnection)
                    {
                        isApiDemonstratorInitialized = true;
                        UpdateAvailabilityStatus();
                        btnConnectAndDisconnect.Enabled = true;
                        LogFeedback(logString + returnValue.ToString());
                        RetrieveDelegates();
                    }
                    else
                    {
                        LogFeedback(logString + returnValue.ToString());
                    }
                }
                else
                {
                    btnConnectAndDisconnect.Text = Properties.Resources.MainForm_btnDisconnectText;
                    UpdateAvailabilityStatus();
                    btnConnectAndDisconnect.Enabled = true;
                    LogFeedback(logString + returnValue.ToString());
                }
            }
        }

        private void btnStartMeeting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                  + Properties.Resources.Comma + "StartMeetingById" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int meetingid = 0;
                    API_ERROR error = API_ERROR.ERROR;

                    if (textBoxMeetingId.Text.Length != 0)
                    {
                        // Validate meeting id
                        if (CheckAndConvertStringValueToInt(textBoxMeetingId.Text, out meetingid))
                        {
                            // Call Api method to start meeting
                            error = controlApi.MeetingApi.StartMeetingById(meetingid);

                            if (error == API_ERROR.NONE)
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnStopMeeting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                                 + Properties.Resources.Comma + "StopMeetingById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int meetingid = 0;

                    if (textBoxMeetingId.Text.Length != 0)
                    {
                        // Validate meeting id
                        if (CheckAndConvertStringValueToInt(textBoxMeetingId.Text, out meetingid))
                        {
                            // Call Api method to stop meeting
                            API_ERROR error = controlApi.MeetingApi.StopMeetingById(meetingid);

                            if (error == API_ERROR.NONE)
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveMeetings_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                     + Properties.Resources.Comma + "RetrieveMeetings" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    int[] meetingIds = null;
                    MEETING_INFO[] meetinginfo = null;
                    API_ERROR error = API_ERROR.ERROR;

                    // Call Api method to retrieve meeting ids
                    error = configApi.MeetingApi.RetrieveMeetings(out meetingIds, out meetinginfo);
                    if (error == API_ERROR.NONE)
                    {
                        StringBuilder stringBuilder = new StringBuilder();
                        // Log data on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (meetingIds.Length > 0)
                        {
                            LogFeedbackInSameLine(logString + "meetingIds" + Properties.Resources.Comma);
                            for (int i = 0; i < meetingIds.Length; i++)
                            {
                                textBoxFeedback.AppendText(meetingIds[i].ToString() + ",");
                                stringBuilder.Append(meetinginfo[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(" meetings, " + stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnSpeakNow_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                               + Properties.Resources.Comma + "SpeakNow" + Properties.Resources.Comma;
            try
            {
                int delegateId = 0;
                int seatId = 0;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;
                    if (textBoxDelegateId.Text.Length > 0 || textBoxSeatId.Text.Length > 0)
                    {
                        if (textBoxDelegateId.Text.Length > 0)
                        {
                            result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                        }
                        if (textBoxSeatId.Text.Length > 0)
                        {
                            result &= CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                        }
                    }
                    else
                    {
                        result = false;
                    }
                    if (result)
                    {
                        // Call Api method to add delegate in speaker list
                        API_ERROR error = controlApi.DiscussionApi.SpeakNow(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnStopSpeaking_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                   + Properties.Resources.Comma + "StopSpeaking" + Properties.Resources.Comma;
            try
            {
                int delegateId = 0;
                int seatId = 0;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;
                    if (textBoxDelegateId.Text.Length > 0 || textBoxSeatId.Text.Length > 0)
                    {
                        if (textBoxDelegateId.Text.Length > 0)
                        {
                            result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                        }
                        if (textBoxSeatId.Text.Length > 0)
                        {
                            result &= CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                        }
                    }
                    else
                    {
                        result = false;
                    }
                    if (result)
                    {
                        // Call Api method to remove delegate from speaker list
                        API_ERROR error = controlApi.DiscussionApi.StopSpeaking(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }


        private void btnRetrieveRegisteredDelegatesForMeeting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.DelegateApiInterface
                                 + Properties.Resources.Comma + "RetrieveRegisteredDelegatesForMeeting" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (textBoxMeetingIdForRetrieveDelegates.Text.Length != 0)
                    {
                        int meetingId = 0;
                        // Validate meeting id
                        if (CheckAndConvertStringValueToInt(textBoxMeetingIdForRetrieveDelegates.Text, out meetingId))
                        {
                            int[] delegateIds = null;
                            PARTICIPATION_INFO[] participationInfo = null;

                            // Call Api method to retrieve delegate ids for given meeting id
                            API_ERROR error = configApi.DelegateApi.RetrieveRegisteredDelegatesForMeeting(meetingId, out delegateIds, out participationInfo);

                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log meeting ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (delegateIds.Length > 0)
                                {
                                    LogFeedbackInSameLine(logString + "delegateIds" + Properties.Resources.Comma);
                                    for (int i = 0; i < delegateIds.Length; i++)
                                    {
                                        textBoxFeedback.AppendText(delegateIds[i].ToString() + ",");
                                        stringBuilder.Append(participationInfo[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(" participationInfo, " + stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnShift_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                    + Properties.Resources.Comma + "Shift" + Properties.Resources.Comma;

            try
            {
                int delegateId = 0;
                int seatId = 0;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;
                    if (textBoxDelegateId.Text.Length > 0 || textBoxSeatId.Text.Length > 0)
                    {
                        if (textBoxDelegateId.Text.Length > 0)
                        {
                            result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                        }
                        if (textBoxSeatId.Text.Length > 0)
                        {
                            result &= CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                        }
                    }
                    else
                    {
                        result = false;
                    }
                    if (result)
                    {
                        // Call Api method to shift first delegate in request list
                        API_ERROR error = controlApi.DiscussionApi.Shift(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }

        }

        private void btnAppendRequestToSpeak_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                 + Properties.Resources.Comma + "AppendRequestToSpeak" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int delegateId = 0;
                    int seatId = 0;
                    bool result = false;
                    if (textBoxDelegateId.Text.Length > 0 && textBoxSeatId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId) && CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                    }
                    else if (textBoxDelegateId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                    }
                    else if (textBoxSeatId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                    }
                    if (result)
                    {
                        // Call Api method to add to request list
                        API_ERROR error = controlApi.DiscussionApi.AppendRequestToSpeak(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnInsertRequestToSpeak_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                + Properties.Resources.Comma + "InsertRequestToSpeak" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;
                    int newDelegateId = 0;
                    int oldDelegateId = 0;
                    int newSeatId = 0;
                    int oldSeatId = 0;
                    if (textBoxNewDelegateId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxNewDelegateId.Text, out newDelegateId);
                    }
                    if (textBoxOldDelegateId.Text.Length > 0 && result)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxOldDelegateId.Text, out oldDelegateId);
                    }
                    if (textBoxNewSeatId.Text.Length > 0 && result)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxNewSeatId.Text, out newSeatId);
                    }
                    if (textBoxOldSeatId.Text.Length > 0 && result)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxOldSeatId.Text, out oldSeatId);
                    }
                    if (textBoxNewDelegateId.Text.Length == 0 && textBoxOldDelegateId.Text.Length == 0 &&
                        textBoxNewSeatId.Text.Length == 0 && textBoxOldSeatId.Text.Length == 0)
                    {
                        result = false;
                    }
                    if (result)
                    {
                        //  Call Api method to insert request
                        API_ERROR error = controlApi.DiscussionApi.InsertRequestToSpeak(newDelegateId, newSeatId, oldDelegateId, oldSeatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveRespondList_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
       + Properties.Resources.Comma + "RetrieveRespondList" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve response speaker list
                    PARTICIPANT[] particpants = null;
                    API_ERROR error = controlApi.DiscussionApi.RetrieveRespondList(out particpants);
                    if (error == API_ERROR.NONE)
                    {
                        // Log participant id on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (particpants != null && particpants.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            for (int i = 0; i < particpants.Length; i++)
                            {
                                stringBuilder.Append(particpants[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }


        private void btnRemoveRequestToRespond_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                                 + Properties.Resources.Comma + "RemoveRequestToRespond" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int delegateId = 0;
                    int seatId = 0;
                    bool result = false;
                    if (textBoxDelegateId.Text.Length > 0 && textBoxSeatId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId) && CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                    }
                    else if (textBoxDelegateId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                    }
                    else if (textBoxSeatId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                    }
                    if (result)
                    {
                        // Call Api method to remove request to respond
                        API_ERROR error = controlApi.DiscussionApi.RemoveRequestToRespond(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRemoveRequestToSpeak_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                               + Properties.Resources.Comma + "RemoveRequestToSpeak" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int delegateId = 0;
                    int seatId = 0;
                    bool result = false;
                    if (textBoxDelegateId.Text.Length > 0 && textBoxSeatId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId) && CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                    }
                    else if (textBoxDelegateId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                    }
                    else if (textBoxSeatId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                    }
                    if (result)
                    {
                        //Call Api method to remove from request list
                        API_ERROR error = controlApi.DiscussionApi.RemoveRequestToSpeak(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveSeatAssignmentForArea_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.DelegateApiInterface
                                + Properties.Resources.Comma + "RetrieveSeatAssignmentForArea" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (textBoxMeetingIdForRetrieveDelegates.Text.Length != 0)
                    {
                        int meetingId = 0;
                        // Validate meeting id
                        if (CheckAndConvertStringValueToInt(textBoxMeetingIdForRetrieveDelegates.Text, out meetingId))
                        {
                            // Call Api method to retrieve seat assignments for given meeting id
                            SEAT_ASSIGNMENT[] seatAssignments;
                            API_ERROR error = configApi.DelegateApi.RetrieveSeatAssignmentForArea(meetingId, 1, out seatAssignments);
                            if (error == API_ERROR.NONE)
                            {
                                // Log participant id on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (seatAssignments != null && seatAssignments.Length > 0)
                                {
                                    StringBuilder stringBuilder = new StringBuilder();
                                    LogFeedbackInSameLine(logString);
                                    for (int i = 0; i < seatAssignments.Length; i++)
                                    {
                                        stringBuilder.Append(seatAssignments[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }

                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnReplaceRequestToSpeak_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                             + Properties.Resources.Comma + "ReplaceRequestToSpeak" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;
                    int delegateId = 0;
                    int withDelegateId = 0;
                    int seatId = 0;
                    int withSeatId = 0;
                    if (textBoxRequestingDelegateId.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxRequestingDelegateId.Text, out delegateId);
                    }
                    if (textBoxWithDelegateId.Text.Length > 0 && result)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxWithDelegateId.Text, out withDelegateId);
                    }
                    if (textBoxRequestingSeatId.Text.Length > 0 && result)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxRequestingSeatId.Text, out seatId);
                    }
                    if (textBoxWithSeatId.Text.Length > 0 && result)
                    {
                        result = CheckAndConvertStringValueToInt(textBoxWithSeatId.Text, out withSeatId);
                    }
                    if (textBoxRequestingDelegateId.Text.Length == 0 && textBoxWithDelegateId.Text.Length == 0 &&
                            textBoxRequestingSeatId.Text.Length == 0 &&
                                textBoxWithSeatId.Text.Length == 0)
                    {
                        result = false;
                    }
                    if (result)
                    {
                        //  Call Api method to replace request
                        API_ERROR error = controlApi.DiscussionApi.ReplaceRequestToSpeak(delegateId, seatId, withDelegateId, withSeatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnStartSessionById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                 + Properties.Resources.Comma + "StartSessionById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int sessionId = 0;
                    API_ERROR error = API_ERROR.ERROR;

                    if (textboxSessionId.Text.Length != 0)
                    {
                        // Validate session id;
                        if (CheckAndConvertStringValueToInt(textboxSessionId.Text, out sessionId))
                        {
                            // Call Api method to start session
                            error = controlApi.MeetingApi.StartSessionById(sessionId);
                            if (error == API_ERROR.NONE)
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnStopSessionById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                 + Properties.Resources.Comma + "StopSessionById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int sessionId = 0;
                    API_ERROR error = API_ERROR.ERROR;

                    if (textboxSessionId.Text.Length != 0)
                    {
                        // Validate session id
                        if (CheckAndConvertStringValueToInt(textboxSessionId.Text, out sessionId))
                        {
                            // Call Api method to stop session
                            error = controlApi.MeetingApi.StopSessionById(sessionId);
                            if (error == API_ERROR.NONE)
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveMeetingSessions_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                    + Properties.Resources.Comma + "RetrieveMeetingSessions" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    int meetingId = 0;
                    int[] sessionIds = null;
                    SESSION_INFO[] sessions = null;
                    API_ERROR error = API_ERROR.ERROR;

                    if (textBoxMeetingIdForSession.Text.Length != 0)
                    {
                        if (CheckAndConvertStringValueToInt(textBoxMeetingIdForSession.Text, out meetingId))
                        {
                            // Call Api method to retrieve session ids
                            error = configApi.MeetingApi.RetrieveMeetingSessions(meetingId, out sessionIds, out sessions);
                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log meeting ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (sessionIds.Length > 0)
                                {
                                    LogFeedbackInSameLine(logString + "sessionIds" + Properties.Resources.Comma);
                                    for (int i = 0; i < sessionIds.Length; i++)
                                    {
                                        textBoxFeedback.AppendText(sessionIds[i].ToString());
                                        if (i < sessionIds.Length)
                                        {
                                            textBoxFeedback.AppendText(",");
                                        }
                                        stringBuilder.Append(sessions[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(" sessions, " + stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRemoveAllRequests_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                  + Properties.Resources.Comma + "RemoveAllRequests" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to remove all requests from the request-list and response-lists
                    API_ERROR error = controlApi.DiscussionApi.RemoveAllRequests();
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnCancelAll_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                    + Properties.Resources.Comma + "CancelAll" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to cancel all active speakers and clear all requests from the request-list and response-lists
                    API_ERROR error = controlApi.DiscussionApi.CancelAll();
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveRequestToRespondList_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
        + Properties.Resources.Comma + "RetrieveRequestToRespondList" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve response request list 
                    PARTICIPANT[] particpants = null;
                    API_ERROR error = controlApi.DiscussionApi.RetrieveRequestToRespondList(out particpants);
                    if (error == API_ERROR.NONE)
                    {
                        // Log participant id on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (particpants != null && particpants.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            for (int i = 0; i < particpants.Length; i++)
                            {
                                stringBuilder.Append(particpants[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveRequestToSpeakList_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                + Properties.Resources.Comma + "RetrieveRequestToSpeakList" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve request list contents
                    PARTICIPANT[] particpants = null;
                    API_ERROR error = controlApi.DiscussionApi.RetrieveRequestToSpeakList(out particpants);
                    if (error == API_ERROR.NONE)
                    {
                        // Log participant id on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (particpants != null && particpants.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            for (int i = 0; i < particpants.Length; i++)
                            {
                                stringBuilder.Append(particpants[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveSpeakerList_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
               + Properties.Resources.Comma + "RetrieveSpeakerList" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve speaker list contents
                    PARTICIPANT[] particpants = null;
                    API_ERROR error = controlApi.DiscussionApi.RetrieveSpeakersList(out particpants);
                    if (error == API_ERROR.NONE)
                    {
                        // Log participant id on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (particpants != null && particpants.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            stringBuilder.Append(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + Properties.Resources.Comma + " ");
                            for (int i = 0; i < particpants.Length; i++)
                            {
                                stringBuilder.Append(particpants[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveNotebookersList_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                + Properties.Resources.Comma + "RetrieveNotebookersList" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve notebookerss list contents
                    NOTEBOOKER_INFO[] notebookers = null;
                    API_ERROR error = controlApi.DiscussionApi.RetrieveNotebookersList(out notebookers);
                    if (error == API_ERROR.NONE)
                    {
                        // Log notebookers on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (notebookers != null && notebookers.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            for (int i = 0; i < notebookers.Length; i++)
                            {
                                stringBuilder.Append(notebookers[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveMeetingGroup_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingGroupApiInterface
                + Properties.Resources.Comma + "RetrieveMeetingGroups" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    // Call Api method to retrieve meeting group content
                    int[] meetingGroupIds = null;
                    MEETING_GROUP_INFO[] meetingGroupInfo = null;
                    API_ERROR error = configApi.MeetingGroupApi.RetrieveMeetingGroups(out meetingGroupIds, out meetingGroupInfo);
                    if (error == API_ERROR.NONE)
                    {
                        StringBuilder stringBuilder = new StringBuilder();
                        // Log meeting ids on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (meetingGroupIds.Length > 0)
                        {
                            LogFeedbackInSameLine(logString + "meetingGroupIds" + Properties.Resources.Comma);
                            for (int i = 0; i < meetingGroupIds.Length; i++)
                            {
                                textBoxFeedback.AppendText(meetingGroupIds[i].ToString() + ",");
                                stringBuilder.Append(meetingGroupInfo[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(" Meeting groups, " + stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveMeetingGroupById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingGroupApiInterface
                                + Properties.Resources.Comma + "RetrieveMeetingGroupById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (txtMeetingGroupById.Text.Length != 0)
                    {
                        int[] meetingGroupIds = null;

                        // Validate meeting groupid
                        if (ParseStringByComma(txtMeetingGroupById.Text, out meetingGroupIds))
                        {
                            // Call Api method to retrieve meeting group
                            MEETING_GROUP_INFO[] meetingGroupInfo = null;
                            API_ERROR error = configApi.MeetingGroupApi.RetrieveMeetingGroupsById(meetingGroupIds, out meetingGroupInfo);
                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log meeting ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (meetingGroupIds.Length > 0)
                                {
                                    LogFeedbackInSameLine(logString + "meetingGroupIds" + Properties.Resources.Comma);
                                    textBoxFeedback.AppendText(txtMeetingGroupById.Text + ",");
                                    for (int i = 0; i < meetingGroupIds.Length; i++)
                                    {
                                        stringBuilder.Append(meetingGroupInfo[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(" Meeting groups, " + stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveGroupTimers_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
               + Properties.Resources.Comma + "RetrieveGroupTimers" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve meeting group timer content
                    SESSION_GROUP_TIMER[] groupTimer;
                    API_ERROR error = controlApi.DiscussionApi.RetrieveGroupTimers(out groupTimer);

                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (groupTimer != null && groupTimer.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            for (int i = 0; i < groupTimer.Length; i++)
                            {
                                stringBuilder.Append(groupTimer[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveDiscussionSettings_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                    + Properties.Resources.Comma + "RetrieveDiscussionSettings" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    DISCUSSION_INFO discussionInfo;
                    // Call Api method to retrieve discussion settings
                    API_ERROR error = controlApi.DiscussionApi.RetrieveDiscussionSettings(out discussionInfo);
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (discussionInfo != null)
                        {
                            SetDiscussionSettings(discussionInfo);
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnSetDiscussionSettings_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                    + Properties.Resources.Comma + "SetDiscussionSettings" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    DISCUSSION_INFO discussionInfo = new DISCUSSION_INFO();
                    if (IsDiscussionSettingsValid())
                    {
                        GetDiscussionSettings(out discussionInfo);
                        // Call Api method to set discussion settings
                        API_ERROR error = controlApi.DiscussionApi.SetDiscussionSettings(discussionInfo);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnCopyMeeting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                    + Properties.Resources.Comma + "CopyMeeting" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    int oldMeetingId = 0;
                    int newMeetingId = 0;

                    API_ERROR error = API_ERROR.ERROR;

                    if (textBoxMeetingIdForSession.Text.Length != 0)
                    {
                        if (CheckAndConvertStringValueToInt(textBoxMeetingIdForSession.Text, out oldMeetingId))
                        {
                            // Call Api method to retrieve session ids
                            error = configApi.MeetingApi.CopyMeeting(oldMeetingId, out newMeetingId);
                            if (error == API_ERROR.NONE)
                            {
                                // Log on successful Copy
                                LogFeedback(logString + apiErrorString + error.ToString());
                                LogFeedback("Copy Successfull. New Meeting Id is: " + newMeetingId.ToString());
                                textBoxFeedback.AppendText("\r\n");
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveWebStreamingLicense_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.SystemApiInterface
        + Properties.Resources.Comma + "RetrieveWebStreamingLicense" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    WEB_STREAMING_LICENSE license = new WEB_STREAMING_LICENSE();
                    // Call Api method to retrieve web streaming license
                    API_ERROR error = configApi.SystemApi.RetrieveWebStreamingLicense(out license);
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        LogFeedback(logString + "WEB_STREAMING_LICENSE: " + license.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }


        }

        private void tabControlAPI_Selected(object sender, TabControlEventArgs e)
        {
            UpdateDiscussionSettings();
        }

        private void btnRetrieveActiveMeetingById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                    + Properties.Resources.Comma + "RetrieveActiveMeetingById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int meetingId = 0;
                    API_ERROR error = API_ERROR.ERROR;
                    // Call Api method to retrieve session ids
                    error = controlApi.MeetingApi.RetrieveActiveMeetingId(out meetingId);
                    if (error == API_ERROR.NONE)
                    {
                        StringBuilder stringBuilder = new StringBuilder();
                        // Log meeting ids on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        LogFeedbackInSameLine(logString + "MeetingId:" + meetingId);
                        textBoxFeedback.AppendText("\r\n");
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveActiveSessionById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                   + Properties.Resources.Comma + "RetrieveActiveSessionById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int sessionId = 0;
                    API_ERROR error = API_ERROR.ERROR;
                    // Call Api method to retrieve session ids
                    error = controlApi.MeetingApi.RetrieveActiveSessionId(out sessionId);
                    if (error == API_ERROR.NONE)
                    {
                        StringBuilder stringBuilder = new StringBuilder();
                        // Log meeting ids on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        LogFeedbackInSameLine(logString + "SessionId:" + sessionId);
                        textBoxFeedback.AppendText("\r\n");
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveDelegatesForActiveMeeting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
        + Properties.Resources.Comma + "RetrieveDelegatesForActiveMeeting" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    // Call Api method to retrieve delegate for active meetings.
                    DELEGATE[] delegates = null;
                    API_ERROR error = controlApi.MeetingApi.RetrieveDelegatesForActiveMeeting(out delegates);
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (delegates != null && delegates.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            foreach (DELEGATE activeMeetingDelegate in delegates)
                            {
                                stringBuilder.Append(activeMeetingDelegate.DelegateId.ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveMicrophoneStatus_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                   + Properties.Resources.Comma + "RetrieveMicrophoneStatus" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    SEAT_MICROPHONE_STATUS[] seatMicStatus;
                    API_ERROR error = API_ERROR.ERROR;
                    // Call Api method to retrieve mic status
                    error = controlApi.DiscussionApi.RetrieveMicrophoneStatus(out seatMicStatus);
                    if (error == API_ERROR.NONE)
                    {
                        // Log participant id on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (seatMicStatus != null && seatMicStatus.Length > 0)
                        {
                            StringBuilder stringBuilder = new StringBuilder();
                            LogFeedbackInSameLine(logString);
                            for (int i = 0; i < seatMicStatus.Length; i++)
                            {
                                stringBuilder.Append(seatMicStatus[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveActiveVoting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
        + Properties.Resources.Comma + "RetrieveActiveVotingId" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int votingId;

                    API_ERROR error = controlApi.VoteApi.RetrieveActiveVotingId(out votingId);

                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        LogFeedbackInSameLine(logString + "VotingId:" + votingId);
                        textBoxFeedback.AppendText("\r\n");
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveVotingsForActiveSession_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
        + Properties.Resources.Comma + "RetrieveVotingsForActiveSession" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int[] votingIds;

                    API_ERROR error = controlApi.VoteApi.RetrieveVotingsForActiveSession(out votingIds);

                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());

                        StringBuilder stringBuilder = new StringBuilder();

                        if (votingIds.Length > 0)
                        {
                            LogFeedbackInSameLine(logString + "Voting Ids" + Properties.Resources.Comma);
                            for (int i = 0; i < votingIds.Length; i++)
                            {
                                stringBuilder.Append(votingIds[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + "\r\n\r\n");
                        }

                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnCopyVoting_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.VoteDataApiInterface
        + Properties.Resources.Comma + "CopyVotingById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int newVotingID;
                    int oldvotingid;

                    if (CheckAndConvertStringValueToInt(txtOldVotingId.Text, out oldvotingid))
                    {
                        API_ERROR error = configApi.VoteApi.CopyVoting(oldvotingid, out newVotingID);

                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                            LogFeedback("Copy Successfull. New Voting Id is: " + newVotingID.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnStartAdhocVoting_Click_1(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
        + Properties.Resources.Comma + "StartAdhocVoting" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int templatid = 0;
                    int answersetid = 0;

                    CheckAndConvertStringValueToInt(comboBoxAnswerSet.Text, out answersetid);
                    CheckAndConvertStringValueToInt(comboBoxVotingTemplate.Text, out templatid);

                    ADHOC_VOTING_SETTINGS setting = new ADHOC_VOTING_SETTINGS();

                    setting.VotingName = textBoxAdhocVotingName.Text;
                    setting.VotingNumber = textBoxAdhocVotingNumber.Text;
                    setting.Subject = textBoxAdhocVotingSubject.Text;
                    setting.AnswerSet = (VotingAnswerSetType)answersetid;
                    setting.VotingTemplateId = templatid;

                    API_ERROR error = controlApi.VoteApi.StartAdhocVoting(setting);

                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnSummon_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
      + Properties.Resources.Comma + "SummonVoting" + Properties.Resources.Comma;

            try
            {
                if (controlApi != null)
                {
                    API_ERROR error = controlApi.VoteApi.SummonVoting();
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }

        }

        private void btnRetrieveVoteTemplatesById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.VoteDataApiInterface
                                + Properties.Resources.Comma + "RetrieveVoteTemplatesById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (textBoxVoteTemplateIds.Text.Length != 0)
                    {
                        int[] voteTemplateIds = null;

                        // Validate voting template id
                        if (ParseStringByComma(textBoxVoteTemplateIds.Text, out voteTemplateIds))
                        {
                            // Call Api method to retrieve voting template 
                            VOTE_TEMPLATE_INFO[] voteTemplateInfo = null;
                            API_ERROR error = configApi.VoteApi.RetrieveVoteTemplatesById(voteTemplateIds, out voteTemplateInfo);
                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log vote template ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (voteTemplateIds.Length > 0)
                                {
                                    LogFeedbackInSameLine(logString + "voteTemplateIds" + Properties.Resources.Comma);
                                    textBoxFeedback.AppendText(textBoxVoteTemplateIds.Text + ",");
                                    for (int i = 0; i < voteTemplateIds.Length; i++)
                                    {
                                        stringBuilder.Append(voteTemplateInfo[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(" Vote Templates, " + stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + apiErrorString + ex.Message);
            }
        }

        private void btnRetrieveVotesById_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.VoteDataApiInterface
                               + Properties.Resources.Comma + "RetrieveVotesById" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (textBoxVoteId.Text.Length != 0)
                    {
                        int[] voteIds = null;

                        // Validate voting template id
                        if (ParseStringByComma(textBoxVoteId.Text, out voteIds))
                        {
                            // Call Api method to retrieve voting 
                            VOTE_INFO[] voteInfo = null;
                            API_ERROR error = configApi.VoteApi.RetrieveVotesById(voteIds, out voteInfo);
                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log vote ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (voteIds.Length > 0)
                                {
                                    LogFeedbackInSameLine(logString + "voteIds" + Properties.Resources.Comma);
                                    textBoxFeedback.AppendText(textBoxVoteId.Text + ",");
                                    for (int i = 0; i < voteIds.Length; i++)
                                    {
                                        stringBuilder.Append(voteInfo[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(" Vote Info, " + stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + apiErrorString + ex.Message);
            }
        }

        private void btnRetrieveVoteTemplates_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.VoteDataApiInterface
                              + Properties.Resources.Comma + "RetrieveVoteTemplates" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    VOTE_TEMPLATE_INFO[] voteTemplateInfo = null;
                    int[] voteTemplateIds = null;
                    API_ERROR error = configApi.VoteApi.RetrieveVoteTemplates(out voteTemplateIds, out voteTemplateInfo);
                    if (error == API_ERROR.NONE)
                    {
                        StringBuilder stringBuilder = new StringBuilder();
                        // Log vote ids on successful retrieval
                        LogFeedback(logString + apiErrorString + error.ToString());
                        if (voteTemplateIds != null && voteTemplateInfo.Length > 0)
                        {
                            LogFeedbackInSameLine(logString + "voteTemplateIds" + Properties.Resources.Comma);
                            foreach (int id in voteTemplateIds)
                            {
                                stringBuilder.Append(id.ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(stringBuilder.ToString() + ",");

                            stringBuilder.Clear();
                            for (int i = 0; i < voteTemplateInfo.Length; i++)
                            {
                                stringBuilder.Append(voteTemplateInfo[i].ToString() + ",");
                            }
                            stringBuilder.Remove(stringBuilder.Length - 1, 1);
                            textBoxFeedback.AppendText(" Vote Templates, " + stringBuilder.ToString() + "\r\n\r\n");
                        }
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }

                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRetrieveVotingScript_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.VoteDataApiInterface
                              + Properties.Resources.Comma + "RetrieveVotingScript" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (textBoxSnId.Text.Length != 0)
                    {
                        int sesssionid = 0;
                        if (CheckAndConvertStringValueToInt(textBoxSnId.Text, out sesssionid))
                        {
                            VOTE_INFO[] voteInfo = null;
                            int[] voteIds = null;
                            API_ERROR error = configApi.VoteApi.RetrieveVotingScript(sesssionid, out voteIds, out voteInfo);
                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log vote ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                                if (voteIds != null && voteInfo.Length > 0)
                                {
                                    LogFeedbackInSameLine(logString + "voteIds" + Properties.Resources.Comma);
                                    foreach (int id in voteIds)
                                    {
                                        stringBuilder.Append(id.ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(stringBuilder.ToString() + ",");

                                    stringBuilder.Clear();
                                    for (int i = 0; i < voteInfo.Length; i++)
                                    {
                                        stringBuilder.Append(voteInfo[i].ToString() + ",");
                                    }
                                    stringBuilder.Remove(stringBuilder.Length - 1, 1);
                                    textBoxFeedback.AppendText(" Vote Info, " + stringBuilder.ToString() + "\r\n\r\n");
                                }
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }

                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + apiErrorString + ex.Message);
            }
        }

        private void btnDeleteVotes_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.VoteDataApiInterface
                              + Properties.Resources.Comma + "DeleteVotes" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    if (textBoxVoteIds.Text.Length != 0)
                    {
                        int[] voteIds = null;

                        // Validate voting id
                        if (ParseStringByComma(textBoxVoteIds.Text, out voteIds))
                        {
                            // Call Api method to Delete votes

                            API_ERROR error = configApi.VoteApi.DeleteVotes(voteIds);
                            if (error == API_ERROR.NONE)
                            {
                                StringBuilder stringBuilder = new StringBuilder();
                                // Log vote ids on successful retrieval
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + apiErrorString + ex.Message);
            }
        }

        private void LoadVotingTemplatesToVotingControlAPICombo()
        {
            if (configApi != null)
            {
                VOTE_TEMPLATE_INFO[] voteTemplateInfo = null;
                int[] voteTemplateIds = null;
                API_ERROR error = configApi.VoteApi.RetrieveVoteTemplates(out voteTemplateIds, out voteTemplateInfo);

                if (error == API_ERROR.NONE)
                {
                    if (voteTemplateIds != null)
                    {
                        comboBoxVotingTemplate.Items.Clear();
                        comboBoxVotingTemplate.Items.Add("-");
                        comboBoxVotingTemplate.DataSource = null;

                        foreach (int i in voteTemplateIds)
                        {
                            comboBoxVotingTemplate.Items.Add(i);
                        }

                        comboBoxVotingTemplate.SelectedIndex = 0;
                        comboBoxAnswerSet.SelectedIndex = 0;
                    }
                }
            }
        }

        private void btnCancelSpeaking_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DiscussionApiInterface
                   + Properties.Resources.Comma + "CancelSpeaking" + Properties.Resources.Comma;
            try
            {
                int delegateId = 0;
                int seatId = 0;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;
                    if (textBoxDelegateId.Text.Length > 0 || textBoxSeatId.Text.Length > 0)
                    {
                        if (textBoxDelegateId.Text.Length > 0)
                        {
                            result = CheckAndConvertStringValueToInt(textBoxDelegateId.Text, out delegateId);
                        }
                        if (textBoxSeatId.Text.Length > 0)
                        {
                            result &= CheckAndConvertStringValueToInt(textBoxSeatId.Text, out seatId);
                        }
                    }
                    else
                    {
                        result = false;
                    }
                    if (result)
                    {
                        // Call Api method to remove delegate from speaker list
                        API_ERROR error = controlApi.DiscussionApi.CancelSpeaking(delegateId, seatId);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnCopySession_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                    + Properties.Resources.Comma + "CopySession" + Properties.Resources.Comma;

            try
            {
                // Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    int oldSessionId = 0;
                    int newSessionId = 0;

                    API_ERROR error = API_ERROR.ERROR;

                    if (textBoxForSessionId.Text.Length != 0)
                    {
                        if (CheckAndConvertStringValueToInt(textBoxForSessionId.Text, out oldSessionId))
                        {
                            // Call Api method to retrieve session ids
                            error = configApi.MeetingApi.CopySession(oldSessionId, out newSessionId);
                            if (error == API_ERROR.NONE)
                            {
                                // Log on successful Copy
                                LogFeedback(logString + apiErrorString + error.ToString());
                                LogFeedback("Copy Successfull. New Session Id is: " + newSessionId.ToString());
                                textBoxFeedback.AppendText("\r\n");
                            }
                            else
                            {
                                LogFeedback(logString + apiErrorString + error.ToString());
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnGetMasterVolume_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DcnSystemApiController
                  + Properties.Resources.Comma + "GetMasterVolume" + Properties.Resources.Comma;
            try
            {
                int masterVolumeCount = 0;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {

                    // Call Api method to remove delegate from speaker list
                    API_ERROR error = controlApi.DcnSystemApi.GetMasterVolume(out masterVolumeCount);
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        LogFeedback(logString + " Master Volume" + Properties.Resources.Comma + masterVolumeCount.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnSetMasterVolume_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DcnSystemApiController
                  + Properties.Resources.Comma + "SetMasterVolume" + Properties.Resources.Comma;
            try
            {
                int masterVolumeCount = 0;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = false;

                    if (txtMasterVolume.Text.Length > 0)
                    {
                        result = CheckAndConvertStringValueToInt(txtMasterVolume.Text, out masterVolumeCount);
                    }


                    if (result)
                    {
                        // Call Api method to remove delegate from speaker list
                        API_ERROR error = controlApi.DcnSystemApi.SetMasterVolume(masterVolumeCount);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnGetMasterMute_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DcnSystemApiController
                  + Properties.Resources.Comma + "GetMasterMute" + Properties.Resources.Comma;
            try
            {
                bool masterMute = false;
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {

                    // Call Api method to remove delegate from speaker list
                    API_ERROR error = controlApi.DcnSystemApi.GetMasterMute(out masterMute);
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                        LogFeedback(logString + " Master Mute" + Properties.Resources.Comma + masterMute.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnSetMasterMute_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DcnSystemApiController
                 + Properties.Resources.Comma + "SetMasterMute" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;

                    if (result)
                    {
                        // Call Api method to remove delegate from speaker list
                        API_ERROR error = controlApi.DcnSystemApi.SetMasterMute(chkMasterMute.Checked);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }

                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnNrOfActivePrios_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DcnSystemApiController
                + Properties.Resources.Comma + "GetNumberofPriosActive" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    bool result = true;

                    if (result)
                    {
                        int keyprio, VirtualPrio;
                        // Call Api method to remove delegate from speaker list
                        API_ERROR error = controlApi.DiscussionApi.GetNumberOfActivePrios(out  keyprio, out VirtualPrio);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                            LogFeedback(logString + " Key Prios = " + keyprio.ToString() + " Virtual key Prio = " + VirtualPrio.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnSetExternalPrio_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.DcnSystemApiController
                 + Properties.Resources.Comma + "SetExternalPrio" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    int participantId = -1, seatId = -1;
                    bool result = true;
                    if (txtbxParticipantId.Text.Length > 0 || txtbxSeatid.Text.Length > 0)
                    {
                        if (txtbxParticipantId.Text.Length > 0)
                        {
                            result = CheckAndConvertStringValueToInt(txtbxParticipantId.Text, out participantId);
                        }
                        if (txtbxSeatid.Text.Length > 0)
                        {
                            result &= CheckAndConvertStringValueToInt(txtbxSeatid.Text, out seatId);
                        }
                    }
                    else
                    {
                        result = false;
                    }

                    if (result)
                    {
                        // Call Api method to remove delegate from speaker list
                        API_ERROR error = controlApi.DiscussionApi.SetChairmanPriority(participantId, seatId, checkBox1.Checked);
                        if (error == API_ERROR.NONE)
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRenameMeetingTitle_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                   + Properties.Resources.Comma + "UpdateMeetingTitle" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;

                    if (txtNewMeetingTitle.Text.Length != 0)
                    {
                        // Call Api method to update meeting title
                        error = configApi.MeetingApi.UpdateMeetingTitle(Convert.ToInt32(textBoxMeetingIdForSession.Text), txtNewMeetingTitle.Text);
                        if (error == API_ERROR.NONE)
                        {
                            // Log on successful Update Meeting Title
                            LogFeedback(logString + apiErrorString + error.ToString());
                            textBoxFeedback.AppendText("\r\n");
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRenameSessionTitle_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                 + Properties.Resources.Comma + "UpdateSessionTitle" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;

                    if (txtNewSessionTitle.Text.Length != 0)
                    {
                        // Call Api method to Update Session Subject
                        error = configApi.MeetingApi.UpdateSessionTitle(Convert.ToInt32(textBoxForSessionId.Text), txtNewSessionTitle.Text);
                        if (error == API_ERROR.NONE)
                        {
                            // Log on successful Rename Session Title
                            LogFeedback(logString + apiErrorString + error.ToString());
                            textBoxFeedback.AppendText("\r\n");
                        }
                        else
                        {
                            LogFeedback(logString + apiErrorString + error.ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnAddMeetingGroup_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
               + Properties.Resources.Comma + "AddMeetingGroup" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;
                    int newMeetingGrpId = -1;
                    error = configApi.MeetingGroupApi.AddMeetingGroup(out newMeetingGrpId);

                    //Log on successful Addition of new meeting group
                    LogFeedback(logString + apiErrorString + error.ToString());
                    LogFeedback(logString + " New meeting Group id =" + newMeetingGrpId);
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnRenameMeetingGroup_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
             + Properties.Resources.Comma + "UpdateMeetingGroup" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;
                    if (txtMeetingGroupById.Text.Length > 0 && (txtNewMeetingGroupTitle.Text.Length != 0 || txtbxNewMeetigngrpDesc.Text.Length > 0))
                    {
                        error = configApi.MeetingGroupApi.UpdateMeetingGroup(Convert.ToInt32(txtMeetingGroupById.Text), txtNewMeetingGroupTitle.Text, txtbxNewMeetigngrpDesc.Text);

                        //Log on successful Addition of new meeting group
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }

        }

        private void btnDelegateAPIAddRequest_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
              + Properties.Resources.Comma + "AddRequestToSpeak" + Properties.Resources.Comma;

            try
            {

                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    if (txtDelegateSessionId.Text.Length > 0 && txtDelegateAPIDelegateId.Text.Length > 0)
                    {

                        PARTICIPATION_INFO[] participationInfo = null;

                        PARTICIPATION_INFO temp = new PARTICIPATION_INFO();
                        temp.AssignedSeatId = Convert.ToInt32(txtDelegateSessionId.Text);
                        temp.VoteWeight = 1;
                        participationInfo = new PARTICIPATION_INFO[1];
                        participationInfo[0] = temp;
                        List<int> deleids = new List<int>();
                        deleids.Add(Convert.ToInt32(txtDelegateAPIDelegateId.Text));
                        error = configApi.DelegateApi.UpdateRegistrationForDelegates( Convert.ToInt32(textBoxMeetingIdForRetrieveDelegates.Text), deleids.ToArray(),participationInfo);

                        LogFeedback(logString + apiErrorString + error.ToString());
                    }

                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnDelegateAPIRemoveReq_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
           + Properties.Resources.Comma + "RemoveRequestToSpeak" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    if (txtDelegateSessionId.Text.Length > 0 && txtDelegateAPIDelegateId.Text.Length > 0)
                    {
                        IDelegateDataApi delegateApi = configApi.DelegateApi;

                        PARTICIPATION_INFO[] participationInfo = null;

                        PARTICIPATION_INFO temp = new PARTICIPATION_INFO();
                        temp.AssignedSeatId = Convert.ToInt32(txtDelegateSessionId.Text);
                        temp.VoteWeight = 1;
                        participationInfo = new PARTICIPATION_INFO[1];
                        participationInfo[0] = temp;
                        List<int> deleids = new List<int>();
                        deleids.Add(Convert.ToInt32(txtDelegateAPIDelegateId.Text));
                        error = configApi.DelegateApi.RegisterDelegatesForMeeting(Convert.ToInt32(textBoxMeetingIdForRetrieveDelegates.Text), deleids.ToArray(), participationInfo);


                        //error = delegateApi.RemoveRequestToSpeak(Convert.ToInt32(txtDelegateSessionId.Text), Convert.ToInt32(txtDelegateAPIDelegateId.Text));
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void btnDelegateAPIRemoveAllreq_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
            + Properties.Resources.Comma + "RemoveAllRequestToSpeak" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    if (txtDelegateSessionId.Text.Length > 0)
                    {
                        IDelegateDataApi delegateApi = configApi.DelegateApi;
                        error = delegateApi.RemoveAllRequestToSpeak(Convert.ToInt32(txtDelegateSessionId.Text));
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        private void BtnUpdateNotebook_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.SeatDataApiInterface
           + Properties.Resources.Comma + "UpdateNotebook" + Properties.Resources.Comma;

            try
            {
                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    if (txtStationSeatId.Text.Length > 0 && txtStationMicModeType.Text.Length>0)
                    {
                        ISeatDataApi seatDataApi = configApi.SeatDataApi;

                        error = seatDataApi.UpdateNotebooker(Convert.ToInt32(txtStationSeatId.Text), (MicrophoneActivationMode)Convert.ToInt32(txtStationMicModeType.Text));
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }


        private void btnReadCard_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
              + Properties.Resources.Comma + "ReadCard" + Properties.Resources.Comma;

            try
            {

                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    int CardNumber, delegateId;
                    error = configApi.DelegateApi.ReadCard(out CardNumber, out delegateId);
                    LogFeedback(logString + apiErrorString + error.ToString());
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(" Card Code {" + CardNumber + "} and Delegate Id { " + delegateId + "}");

                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }
        private void btnProduceCard_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ConfigApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingDataApiInterface
                         + Properties.Resources.Comma + "ProduceCard" + Properties.Resources.Comma;

            try
            {

                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    if (string.Empty != txtProduceCard.Text.Trim())
                    {
                        error = configApi.DelegateApi.ProduceCardForDelegate(Convert.ToInt32(txtProduceCard.Text));
                    }
                    LogFeedback(logString + apiErrorString + error.ToString());
                     
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }


        private void btnInsertCard_Click(object sender, EventArgs e)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.MeetingApiInterface
                        + Properties.Resources.Comma + "InsertCard" + Properties.Resources.Comma;

            try
            {

                //Check if a connection to the DCN-SW server is available
                if (configApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    if (string.Empty != txtUnitId.Text.Trim() && string.Empty != txtCardCode.Text.Trim())
                    {
                        error = controlApi.MeetingApi.InsertCard(Convert.ToInt32(txtUnitId.Text), Convert.ToInt32(txtCardCode.Text));
                    }
                    LogFeedback(logString + apiErrorString + error.ToString());

                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Start the voting
        /// </summary>
        /// <param name="votingId">Voting Id.</param>
        private void StartVoting(int votingId)
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
                + Properties.Resources.Comma + "StartVotingById" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;
                    error = controlApi.VoteApi.StartVotingById(votingId);
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        /// <summary>
        /// Stop the voting.
        /// </summary>
        private void StopVoting()
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
                 + Properties.Resources.Comma + "StopVoting" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {
                    API_ERROR error = API_ERROR.ERROR;
                    error = controlApi.VoteApi.StopVoting();
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        /// <summary>
        /// Puts the current active voting on hold.
        /// </summary>
        private void HoldVoting()
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
                 + Properties.Resources.Comma + "HoldVoting" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;
                    error = controlApi.VoteApi.HoldVoting();
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }

        /// <summary>
        /// Resumes the Voting.
        /// </summary>
        private void ResumeVoting()
        {
            string logString = Properties.Resources.ControlApiInterface + Properties.Resources.Comma + Properties.Resources.VoteApiInterface
                 + Properties.Resources.Comma + "ResumeVoting" + Properties.Resources.Comma;
            try
            {
                // Check if a connection to the DCN-SW server is available
                if (controlApi != null)
                {

                    API_ERROR error = API_ERROR.ERROR;
                    error = controlApi.VoteApi.ContinueVoting();
                    if (error == API_ERROR.NONE)
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                    else
                    {
                        LogFeedback(logString + apiErrorString + error.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(logString + ex.Message);
            }
        }
        /// <summary>
        /// Updates availability state of Api
        /// </summary>
        private void UpdateAvailabilityStatus()
        {
            try
            {
                if (isApiDemonstratorInitialized && isApiAvailable)
                {
                    lblIsAvailable.BackColor = Color.LightGreen;
                    tabAPI.Enabled = true;
                    btnConnectAndDisconnect.Text = Properties.Resources.MainForm_btnDisconnectText;
                    LoadVotingTemplatesToVotingControlAPICombo();
                }
                else
                {
                    lblIsAvailable.BackColor = Color.LightCoral;
                }

                if ((configApi != null) && (configApi.IsAvailable) && (configApi.IsConfigAllowed)
                    && (isApiDemonstratorInitialized) && isApiAvailable)
                {
                    lblIsConfigAllowed.BackColor = Color.LightGreen;
                    tabConfigAPI.Enabled = true;
                }
                else
                {
                    lblIsConfigAllowed.BackColor = Color.LightCoral;
                }

                if ((controlApi != null) && (controlApi.IsAvailable) && (controlApi.IsControlAllowed)
                    && (isApiDemonstratorInitialized) && isApiAvailable)
                {
                    lblIsControlAllowed.BackColor = Color.LightGreen;
                    tabControlAPI.Enabled = true;
                }
                else
                {
                    lblIsControlAllowed.BackColor = Color.LightCoral;
                }
            }
            catch (Exception ex)
            {
                LogFeedback(ex.Message);
            }
        }

        /// <summary>
        /// Clears notification logs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClearNotification_Click(object sender, EventArgs e)
        {
            textBoxNotification.Clear();
        }

        /// <summary>
        /// logs function call feedback
        /// </summary>
        /// <param name="s">String value to log</param>
        private void LogFeedback(string s)
        {
            string dateTime = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString();
            string logString = dateTime + Properties.Resources.Comma + Properties.Resources.MainForm_textBoxFeedback_ApiText + Properties.Resources.Comma;
            textBoxFeedback.AppendText(logString + s + "\r\n");
        }

        /// <summary>
        /// logs function call feedback
        /// </summary>
        /// <param name="s">String value to log</param>
        private void LogFeedbackInSameLine(string s)
        {
            string dateTime = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString();
            string logString = dateTime + Properties.Resources.Comma + Properties.Resources.MainForm_textBoxFeedback_ApiText + Properties.Resources.Comma;
            textBoxFeedback.AppendText(logString + s);
        }

        /// <summary>
        /// logs function call feedback
        /// </summary>
        /// <param name="s">String value to log</param>
        private void LogFeedbackWithDate(string s)
        {
            string dateTime = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString();
            string logString = dateTime + Properties.Resources.Comma + " ";
            textBoxFeedback.AppendText(logString + s + "\r\n");
        }

        /// <summary>
        /// Logs event notifications
        /// </summary>
        /// <param name="s">String value to log</param>
        public void LogNotification(string s)
        {
            string dateTime = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + Properties.Resources.Comma;
            textBoxNotification.AppendText(dateTime + s + "\r\n");
        }

        /// <summary>
        /// Resets scrollbars position on resize of main form
        /// </summary>
        /// <param name="sender">Main form object</param>
        /// <param name="e">Event arguments</param>
        private void APIDemonstratorMainForm_Resize(object sender, EventArgs e)
        {
            this.HorizontalScroll.Value = 0;
            this.VerticalScroll.Value = 0;
        }

        /// <summary>
        /// Checks and converts string to integer value
        /// </summary>
        /// <param name="text">string number</param>
        /// <param name="value">converted interger number</param>
        /// <returns></returns>
        private bool CheckAndConvertStringValueToInt(string text, out int value)
        {
            try
            {
                value = int.Parse(text);
                return true;
            }
            catch
            {
                value = 0;
                return false;
            }
        }

        /// <summary>
        /// Checks validity of the string value
        /// </summary>
        /// <param name="text">string number</param>
        /// <param name="value">converted interger number</param>
        /// <returns></returns>
        private bool CheckStringValue(string text)
        {
            int value = 0;
            try
            {
                value = int.Parse(text);
                return true;
            }
            catch
            {
                return false;
            }
        }
        /// <summary>
        /// Retrieves active meeting id
        /// </summary>
        /// <returns></returns>
        private int ActiveMeetingId()
        {
            int meetingid = 0;
            if (configApi.IsAvailable && configApi.IsConfigAllowed)
            {
                if (controlApi.IsAvailable && controlApi.IsControlAllowed)
                {
                    API_ERROR error;
                    error = controlApi.MeetingApi.RetrieveActiveMeetingId(out meetingid);
                }
            }
            return meetingid;
        }

        /// <summary>
        /// Retrieves active meeting id
        /// </summary>
        /// <returns></returns>
        private int ActiveSessionId()
        {
            int sessionid = 0;
            if (configApi.IsAvailable && configApi.IsConfigAllowed)
            {
                if (controlApi.IsAvailable && controlApi.IsControlAllowed)
                {
                    API_ERROR error;
                    error = controlApi.MeetingApi.RetrieveActiveSessionId(out sessionid);
                }
            }
            return sessionid;
        }

        /// <summary>
        /// Checks and converts string array to integer array value
        /// </summary>
        /// <param name="text"></param>
        /// <param name="valueArray"></param>
        /// <returns></returns>
        private bool ParseStringByComma(string text, out int[] valueArray)
        {
            try
            {
                string[] stringArray = Regex.Split(text, ",");
                valueArray = new int[stringArray.Length];
                for (int i = 0; i < stringArray.Length; i++)
                {
                    valueArray[i] = int.Parse(stringArray[i]);
                }
                return true;
            }
            catch
            {
                valueArray = null;
                return false;
            }
        }

        private void SetDiscussionSettings(DISCUSSION_INFO discussionSettings)
        {
            txtBoxActiveMicrosCount.Text = discussionSettings.NumberOfOpenMicrophones.ToString();
            txtBoxMaxRequestsCount.Text = discussionSettings.MaximumNumberOfRequests.ToString();
            textBoxMaxRespRequest.Text = discussionSettings.MaximumNumberOfRequestToRespond.ToString();
            txtBoxPriorityTone.Text = discussionSettings.AttentionTone.ToString();
            chkBoxSwitchMicrosOffAfterShift.Checked = discussionSettings.MicrophonesOffAfterShift;
            chkBoxParticipantCanCancelRequest.Checked = discussionSettings.AllowCancelRequest;
            chkBoxParticipantCanSwitchMicroOff.Checked = discussionSettings.AllowMicrophoneOff;
            chkBoxAutomaticMicroOff.Checked = discussionSettings.AutomaticMicrophoneOff;
            chkBoxAmbientMicroControl.Checked = discussionSettings.AmbientMicrophone;
            chkBoxCancelAllOnChairmanPriority.Checked = discussionSettings.CancelAllSpeakersAndRequests;
            switch (discussionSettings.MicControlMode)
            {
                case CONTROL_MODE_OPEN: txtBoxMicroControlMode.Text = CONTROL_MODE_OPEN.ToString(); break;
                case CONTROL_MODE_OPERATOR: txtBoxMicroControlMode.Text = CONTROL_MODE_OPERATOR.ToString(); break;
                case CONTROL_MODE_OVERRIDE: txtBoxMicroControlMode.Text = CONTROL_MODE_OVERRIDE.ToString(); break;
                case CONTROL_MODE_PUSHTOTALK: txtBoxMicroControlMode.Text = CONTROL_MODE_PUSHTOTALK.ToString(); break;
                case CONTROL_MODE_RESPONSE: txtBoxMicroControlMode.Text = CONTROL_MODE_RESPONSE.ToString(); break;
                case CONTROL_MODE_VOICE: txtBoxMicroControlMode.Text = CONTROL_MODE_VOICE.ToString(); break;
            }
        }

        private void GetDiscussionSettings(out DISCUSSION_INFO discussionInfo)
        {
            DISCUSSION_INFO discussionSettings = new DISCUSSION_INFO();
            discussionSettings.NumberOfOpenMicrophones = int.Parse(txtBoxActiveMicrosCount.Text);
            discussionSettings.MaximumNumberOfRequests = int.Parse(txtBoxMaxRequestsCount.Text);
            discussionSettings.MaximumNumberOfRequestToRespond = int.Parse(textBoxMaxRespRequest.Text);
            discussionSettings.AttentionTone = int.Parse(txtBoxPriorityTone.Text);
            discussionSettings.MicrophonesOffAfterShift = chkBoxSwitchMicrosOffAfterShift.Checked;
            discussionSettings.AllowCancelRequest = chkBoxParticipantCanCancelRequest.Checked;
            discussionSettings.AllowMicrophoneOff = chkBoxParticipantCanSwitchMicroOff.Checked;
            discussionSettings.AutomaticMicrophoneOff = chkBoxAutomaticMicroOff.Checked;
            discussionSettings.AmbientMicrophone = chkBoxAmbientMicroControl.Checked;
            discussionSettings.CancelAllSpeakersAndRequests = chkBoxCancelAllOnChairmanPriority.Checked;
            int microphonelMode = int.Parse(txtBoxMicroControlMode.Text);
            discussionSettings.MicControlMode = microphonelMode;
            discussionInfo = discussionSettings;
        }

        private bool IsDiscussionSettingsValid()
        {
            if (CheckStringValue(txtBoxActiveMicrosCount.Text) &&
                CheckStringValue(txtBoxMaxRequestsCount.Text) &&
                CheckStringValue(textBoxMaxRespRequest.Text) &&
                CheckStringValue(txtBoxMicroControlMode.Text) &&
                CheckStringValue(txtBoxPriorityTone.Text))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private void UpdateDiscussionSettings()
        {
            if (controlApi != null)
            {
                if (controlApi.IsAvailable && controlApi.IsControlAllowed)
                {
                    discussionSettingsValue = new DISCUSSION_INFO();
                    API_ERROR error = controlApi.DiscussionApi.RetrieveDiscussionSettings(out discussionSettingsValue);
                    if (error == API_ERROR.NONE)
                    {
                        SetDiscussionSettings(discussionSettingsValue);
                    }
                }
            }
        }

        #endregion

        #region Fields

        public IConfigApi configApi;
        public IControlApi controlApi;
        private delegate API_ERROR AsyncConnector(string ConnectionString, string username, string password);
        private bool isApiDemonstratorInitialized;
        bool isInConnection = false;
        string apiErrorString = Properties.Resources.ApiErrorText + Properties.Resources.Period;
        bool isApiAvailable = false;

        DISCUSSION_INFO discussionSettingsValue;
        // Micrphone mode values
        private const int CONTROL_MODE_OPEN = 1;
        private const int CONTROL_MODE_OPERATOR = 3;
        private const int CONTROL_MODE_OVERRIDE = 2;
        private const int CONTROL_MODE_PUSHTOTALK = 6;
        private const int CONTROL_MODE_RESPONSE = 4;
        private const int CONTROL_MODE_VOICE = 5;
        #endregion

        #region Example Tab Region

        /// <summary>
        ///  retrieves the delegates from DB 
        /// </summary>
        private void RetrieveDelegates()
        {
            try
            {
                // Check if a connection to the DCN-SW server is available.
                if (configApi.IsAvailable)
                {
                    // Check whether the authorization to use the DCN-SW API is valid.
                    if (configApi.IsConfigAllowed)
                    {
                        // Clear the dataset bound to the datagrid.
                        DelegateDataset.DelegateDataTable.Clear();

                        // retrieve the delegates
                        int[] delegateIds;
                        DELEGATE_INFO[] delegates;
                        API_ERROR error = configApi.DelegateApi.RetrieveDelegates(out delegateIds, out delegates);

                        if (API_ERROR.NONE == error)
                        {
                            if (delegates.Length != 0)
                            {
                                // retrieve the delegate card assignments
                                int[] cardIds;
                                error = configApi.DelegateApi.RetrieveDelegateIdCardAssignments(delegateIds, out cardIds);

                                if (API_ERROR.NONE == error)
                                {
                                    if (delegates.Length == cardIds.Length)
                                    {
                                        for (int i = 0; i < delegates.Length; i++)
                                        {
                                            DataRow dr = DelegateDataset.DelegateDataTable.NewRow();
                                            dr["Id"] = delegateIds[i];
                                            dr["Title"] = delegates[i].Title;
                                            dr["DisplayLanguageId"] = delegates[i].DisplayLanguageId;
                                            dr["FirstName"] = delegates[i].FirstName;
                                            dr["LastName"] = delegates[i].LastName;
                                            dr["GroupId"] = delegates[i].MeetingGroupId;
                                            dr["MiddleName"] = delegates[i].MiddleName;
                                            dr["PinCode"] = delegates[i].PinCode;
                                            dr["Country"] = delegates[i].Country;
                                            dr["NativeLanguageId"] = delegates[i].NativeLanguageId;
                                            dr["SpokenLanguageId"] = delegates[i].SpokenLanguageId;
                                            dr["Region"] = delegates[i].Region;
                                            dr["CardId"] = cardIds[i];
                                            DelegateDataset.DelegateDataTable.Rows.Add(dr);
                                        }
                                    }
                                    else
                                    {
                                        LogFeedback("Unable to retrieve delegates list from DCN system");
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback("exception occured while retriving the delegates from DCN "+ex.ToString());
            }
        }

        /// <summary>
        /// Removes delegates from DB
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnremovedelegates_Click(object sender, EventArgs e)
        {
            try
            { // Check if a connection to the DCN-SW server is available.
                if (configApi.IsAvailable)
                {
                    // Check whether the authorization to use the DCN-SW API is valid.
                    if (configApi.IsConfigAllowed)
                    {
                        // Clear the dataset bound to the datagrid.
                        DelegateDataset.DelegateDataTable.Clear();

                        // retrieve the delegates
                        int[] delegateIds;
                        DELEGATE_INFO[] delegates;
                        API_ERROR error = configApi.DelegateApi.RetrieveDelegates(out delegateIds, out delegates);

                        if (API_ERROR.NONE == error)
                        {
                            if (delegates.Length != 0)
                            {
                                error = configApi.DelegateApi.DeleteDelegates(delegateIds);

                                if (API_ERROR.NONE == error)
                                {
                                    LogFeedback("sucessfully removed all the delegates from DCN System");
                                }
                                else
                                {
                                    LogFeedback("Error while removing delegates "+error.ToString());
                                }
                            }
                        }
                    }
                }
               
            }
            catch (Exception ex)
            {
                LogFeedback("Exception occured while removing delegates" + ex.ToString());
            }
            RetrieveDelegates();
        }

        /// <summary>
        ///  Export to excel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExporttoexcel_Click(object sender, EventArgs e)
        {
            // Check if a connection to the DCN-SW server is available.
            if (configApi.IsAvailable)
            {
                // Check whether the authorization to use the DCN-SW API is valid.
                if (configApi.IsConfigAllowed)
                {
                    // Get excel file name from user
                    if (SaveExcelFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        bool isexceptionOccured = false;
                        try
                        {
                            // Remove if the file already exists
                            File.Delete(SaveExcelFileDialog.FileName);
                        }
                        catch (System.IO.IOException ex)
                        {
                            isexceptionOccured = true;
                            LogFeedback("Exception occured while deleting existing excel {0} " + ex.ToString());
                        }
                        if (!isexceptionOccured)
                        {
                            OleDbConnection conn;
                            using (conn = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + SaveExcelFileDialog.FileName + ";Extended Properties='Excel 8.0;HDR=Yes'"))
                            {
                                conn.Open();
                                // Create the Excel Delegates table
                                OleDbCommand cmd = new OleDbCommand("CREATE TABLE [Delegates]([Id] integer, [GroupId] integer, [FirstName] string, [MiddleName] string, [LastName] string, [Title] string, [Country] string, [Region] string, [Pincode] string, [DisplayLanguageId] integer, [NativeLanguageId] integer, [SpokenLanguageId] integer, [CardId] integer)", conn);
                                try
                                {
                                    cmd.ExecuteNonQuery();
                                    // Fill the Excel Delegates table
                                    foreach (DataRow dr in DelegateDataset.DelegateDataTable.Rows)
                                    {
                                        string temp = "INSERT INTO [Delegates$] (Id, GroupId, FirstName, MiddleName, LastName, Title, Country, Region, Pincode, DisplayLanguageId, NativeLanguageId, SpokenLanguageId, CardId) VALUES ("
                                            + dr["Id"] + ","
                                            + dr["GroupId"] + ","
                                            + "\"" + dr["FirstName"].ToString() + "\","
                                            + "\"" + dr["MiddleName"].ToString() + "\","
                                            + "\"" + dr["LastName"].ToString() + "\","
                                            + "\"" + dr["Title"].ToString() + "\","
                                            + "\"" + dr["Country"].ToString() + "\","
                                            + "\"" + dr["Region"].ToString() + "\","
                                            + "\"" + dr["Pincode"].ToString() + "\","
                                            + dr["DisplayLanguageId"] + ","
                                            + dr["NativeLanguageId"] + ","
                                            + dr["SpokenLanguageId"] + ","
                                            + dr["CardId"]
                                            + ")";
                                        cmd = new OleDbCommand(temp, conn);
                                        cmd.ExecuteNonQuery();
                                    }
                                    // Close the Excel file

                                }
                                catch (Exception ex)
                                {
                                    LogFeedback("Exception occured while exporting to excel {0} " + ex.ToString());
                                }
                                finally
                                {
                                    conn.Close();
                                }

                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// throws error to user for format exception.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView1_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            try
            { }
            catch (Exception ex)
            {
                LogFeedback("Exception occured while editing {0} " + ex.ToString());

            }
        }

        /// <summary>
        /// udpate the delegate data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnupdatedelegates_Click(object sender, EventArgs e)
        {
            try
            {
                API_ERROR error = API_ERROR.ERROR;
                // Check if a connection to the DCN-SW server is available.
                if (configApi.IsAvailable)
                {
                    // Check whether the authorization to use the DCN-SW API is valid.
                    if (configApi.IsConfigAllowed)
                    {
                        if (configApi.DelegateApi.IsDelegateRegistrationAllowed)
                        {
                            List<DELEGATE_INFO> lstDelegateInfo = new List<DELEGATE_INFO>();
                            List<int> lstCardids = new List<int>();
                            List<int> lstDelegateIds = new List<int>();
                            foreach (DataRow dr in DelegateDataset.DelegateDataTable.Rows)
                            {
                                DELEGATE_INFO delegateinfo = new DELEGATE_INFO();
                                delegateinfo.MeetingGroupId = Convert.ToInt32(dr["GroupId"]);
                                delegateinfo.Country = Convert.ToString(dr["Country"]);
                                delegateinfo.FirstName = Convert.ToString(dr["FirstName"]);
                                delegateinfo.MiddleName = Convert.ToString(dr["MiddleName"]);
                                delegateinfo.LastName = Convert.ToString(dr["LastName"]);
                                delegateinfo.Title = Convert.ToString(dr["Title"]);
                                delegateinfo.Region = Convert.ToString(dr["Region"]);
                                delegateinfo.PinCode = Convert.ToInt32(dr["Pincode"]);
                                delegateinfo.SpokenLanguageId = Convert.ToInt32(dr["SpokenLanguageId"]);
                                delegateinfo.DisplayLanguageId = Convert.ToInt32(dr["DisplayLanguageId"]);
                                delegateinfo.NativeLanguageId = Convert.ToInt32(dr["NativeLanguageId"]);
                                lstDelegateInfo.Add(delegateinfo);
                                lstCardids.Add(Convert.ToInt32(dr["CardId"]));
                                lstDelegateIds.Add(Convert.ToInt32(dr["Id"]));
                            }
                            if (lstDelegateInfo.Count > 0 && lstDelegateIds.Count > 0 && lstDelegateIds.Count == lstDelegateInfo.Count)
                            {
                                error = configApi.DelegateApi.UpdateDelegates(lstDelegateIds.ToArray(), lstDelegateInfo.ToArray());

                                if (error == API_ERROR.NONE)
                                {
                                    LogFeedback(string.Format("updated {0} delegates !", lstDelegateInfo.Count));
                                    if (error == API_ERROR.NONE && lstCardids.ToArray().Length > 0 && lstDelegateInfo.Count > 0 && lstCardids.ToArray().Length == lstDelegateIds.Count)
                                    {
                                        error = configApi.DelegateApi.AssignDelegateIdCards(lstDelegateIds.ToArray(), lstCardids.ToArray());

                                        if (API_ERROR.NONE != error)
                                        {
                                            LogFeedback(string.Format("assigning cards failed {0}!", error));
                                        }
                                    }
                                    else
                                    {
                                        LogFeedback(string.Format("The number of cards {0} and delegates {1} do not match!", lstCardids.ToArray().Length, lstDelegateIds.Count));
                                    }
                                }
                                else
                                {
                                    LogFeedback(string.Format("update {0} delegates failed !", lstDelegateInfo.Count));
                                }
                            }
                        }
                    }
                  
                }
            }
            catch (Exception ex)
            {
                LogFeedback(string.Format("Exception occured while updating delegates ", ex.ToString()));
            }
            RetrieveDelegates();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnappendtoDCN_Click(object sender, EventArgs e)
        {
            try
            {
                API_ERROR error = API_ERROR.ERROR;
                // Check if a connection to the DCN-SW server is available.
                if (configApi.IsAvailable)
                {
                    // Check whether the authorization to use the DCN-SW API is valid.
                    if (configApi.IsConfigAllowed)
                    {
                        if (configApi.DelegateApi.IsDelegateRegistrationAllowed)
                        {
                            List<DELEGATE_INFO> lstDelegateInfo = new List<DELEGATE_INFO>();
                            List<int> lstCardids = new List<int>();
                            foreach (DataRow dr in DelegateDataset.DelegateDataTable.Rows)
                            {
                                DELEGATE_INFO delegateinfo = new DELEGATE_INFO();
                                delegateinfo.MeetingGroupId = Convert.ToInt32(dr["GroupId"]);
                                delegateinfo.Country = Convert.ToString(dr["Country"]);

                                delegateinfo.FirstName = Convert.ToString(dr["FirstName"]);
                                delegateinfo.MiddleName = Convert.ToString(dr["MiddleName"]);
                                delegateinfo.LastName = Convert.ToString(dr["LastName"]);
                                delegateinfo.Title = Convert.ToString(dr["Title"]);
                                delegateinfo.Region = Convert.ToString(dr["Region"]);
                                delegateinfo.PinCode = Convert.ToInt32(dr["Pincode"]);
                                delegateinfo.SpokenLanguageId = Convert.ToInt32(dr["SpokenLanguageId"]);
                                delegateinfo.DisplayLanguageId = Convert.ToInt32(dr["DisplayLanguageId"]);
                                delegateinfo.NativeLanguageId = Convert.ToInt32(dr["NativeLanguageId"]);
                                lstDelegateInfo.Add(delegateinfo);
                                lstCardids.Add(Convert.ToInt32(dr["CardId"]));
                            }
                            int[] arrdelids = new int[0];
                            if (lstDelegateInfo.Count > 0)
                            {
                                error = configApi.DelegateApi.StoreDelegates(lstDelegateInfo.ToArray(), out arrdelids);

                                if (error == API_ERROR.NONE)
                                {
                                    LogFeedback(string.Format("stored  {0} delegates !", lstDelegateInfo.Count));
                                    // Assign the cardIds for the delegates
                                    if (lstCardids.ToArray().Length > 0 && lstDelegateInfo.Count > 0 && lstCardids.ToArray().Length == arrdelids.Length)
                                    {
                                        error = configApi.DelegateApi.AssignDelegateIdCards(arrdelids, lstCardids.ToArray());

                                        if (API_ERROR.NONE != error)
                                        {
                                            LogFeedback(string.Format("assigning cards failed {0}!", error));
                                        }
                                    }
                                    else
                                    {
                                        LogFeedback(string.Format("The number of cards {0} and delegates {1} do not match!", lstCardids.ToArray().Length, arrdelids.Length));
                                    }
                                }
                                else
                                {
                                    LogFeedback(string.Format("stored  {0} delegates failed !", lstDelegateInfo.Count));
                                }
                            }
                        }
                        else
                        {
                            LogFeedback("Delegate registration is not allowed");
                        }
                        RetrieveDelegates();
                    }
                    else
                    {
                        LogFeedback("Config not allowed");
                    }
                }
            }
            catch (Exception ex)
            {
                LogFeedback(string.Format("Exception occured while appending to DCN {0} ", ex.ToString()));
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ImportfromExcel(object sender, EventArgs e)
        {
            try
            {
                // Check if a connection to the DCN-SW server is available.
                if (configApi.IsAvailable)
                {
                    // Check whether the authorization to use the DCN-SW API is valid.
                    if (configApi.IsConfigAllowed)
                    {
                        // Get excel file name from user
                        if (ofdSelectExcelFile.ShowDialog() == DialogResult.OK)
                        {
                            // Remove if the file already exists
                            if (File.Exists(ofdSelectExcelFile.FileName))
                            {
                                string path = System.IO.Path.GetFullPath(ofdSelectExcelFile.FileName);
                                OleDbConnection conn = null;

                                if (Path.GetExtension(path).ToLower().Equals(".xls"))
                                {
                                    conn = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + path + ";Extended Properties=\"Excel 8.0;HDR=Yes;IMEX=2\"");
                                }
                                else if (Path.GetExtension(path).ToLower().Equals(".xlsx"))
                                {
                                    conn = new OleDbConnection(@"Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + path + ";Extended Properties='Excel 12.0;HDR=YES;IMEX=1;';");
                                }
                                DataSet ds = new DataSet();
                                try
                                {
                                    conn.Open();
                                    string query = string.Format("SELECT * FROM [{0}]", "Delegates$");
                                    DelegateDataset.DelegateDataTable.Clear();
                                    OleDbDataAdapter myImportCommand = new OleDbDataAdapter(query, conn);
                                    myImportCommand.Fill(ds);
                                }
                                catch (Exception ex)
                                {
                                    LogFeedback("Exception occurred while importing from excel" +ex.ToString());
                                }
                                finally
                                {
                                    if (null != conn && conn.State == ConnectionState.Open)
                                    {
                                        conn.Close();
                                    }
                                    DelegateDataset.DelegateDataTable.Clear();
                                    if (ds != null && ds.Tables.Count > 0)
                                    {
                                        foreach (DataRow datarow in ds.Tables[0].Rows)
                                        {

                                            DataRow dr = DelegateDataset.DelegateDataTable.NewRow();
                                            if ( datarow["CardId"]!=DBNull.Value&& datarow["Id"] != DBNull.Value && datarow["DisplayLanguageId"] != DBNull.Value && datarow["PinCode"] !=DBNull.Value && datarow["GroupId"] !=DBNull.Value && datarow["NativeLanguageId"] !=DBNull.Value && datarow["SpokenLanguageId"]!= DBNull.Value)
                                            {
                                                dr["Id"] = Convert.ToInt32(datarow["Id"]);
                                                dr["Title"] = Convert.ToString(datarow["Title"]);
                                                dr["DisplayLanguageId"] = Convert.ToString(datarow["DisplayLanguageId"]);
                                                dr["FirstName"] = Convert.ToString(datarow["FirstName"]);
                                                dr["LastName"] = Convert.ToString(datarow["LastName"]);
                                                dr["GroupId"] = Convert.ToInt32(datarow["GroupId"]);
                                                dr["MiddleName"] = Convert.ToString(datarow["MiddleName"]);
                                                dr["PinCode"] = Convert.ToInt32(datarow["PinCode"]);
                                                dr["Country"] = Convert.ToString(datarow["Country"]);
                                                dr["NativeLanguageId"] = Convert.ToInt32(datarow["NativeLanguageId"]);
                                                dr["SpokenLanguageId"] = Convert.ToInt32(datarow["SpokenLanguageId"]);
                                                dr["Region"] = Convert.ToString(datarow["Region"]);
                                                dr["CardId"] = Convert.ToInt32(datarow["CardId"]);
                                                DelegateDataset.DelegateDataTable.Rows.Add(dr);
                                            }
                                        }
                                       
                                        LogFeedback(" Delegate data imported from excel to UI ");
                                    }
                                    DelegateDatagridview.Refresh();
                                }

                            }
                        }
                    }

                }
            }
            catch (Exception ex)
            {
                LogFeedback(string.Format("Exception occured while importing form excel {0}", ex.ToString()));
            }
        }

        /// <summary>
        /// data grid data error 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView1_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            try
            {
                LogFeedback("Exception occured while editing {0} " + e.Exception);
            }
            catch (Exception ex)
            {
                LogFeedback("Exception occured while editing {0} " + ex.ToString());

            }
        }
        #endregion

    }
}
